# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QTableView, QWidget


class Foo(QTableView):
    def __init__(self, parent=None):
        QTableView.__init__(self, parent)


from helper.usesqapplication import UsesQApplication


class TestParentType(UsesQApplication):

    def testParentType(self):
        # Test the problem with calling QObject.parent from a QWidget
        # when the parent is a python class derived from a QWidget-derived
        # class. The method was returning the last C++ class in the hierarchy
        parent = Foo()
        w2 = QWidget(parent)
        self.assertTrue(isinstance(w2.parentWidget(), Foo))
        self.assertTrue(isinstance(w2.parent(), Foo))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QTimer, Slot


class MyTimer (QTimer):
    def __init__(self):
        super().__init__()
        self.startCalled = False

    @Slot()
    def slotUsedToIncreaseMethodOffset(self):
        pass


class MyTimer2 (MyTimer):

    @Slot()
    def slotUsedToIncreaseMethodOffset2(self):
        pass

    def start(self):
        """ 'Overrides' non-virtual slot QTimer.start()."""
        self.startCalled = True
        QCoreApplication.instance().quit()


class TestBug1019 (unittest.TestCase):
    def testIt(self):
        app = QCoreApplication([])
        t = MyTimer2()
        QTimer.singleShot(0, t.start)
        app.exec()
        self.assertTrue(t.startCalled)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtQml import QmlElement
from PySide6.QtCore import QObject, Slot, Property, Signal, QUrl
from PySide6.QtMultimedia import QSoundEffect

from autogen.settings import project_root

QML_IMPORT_NAME = "Audio"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class AudioEngine(QObject):
    volumeChanged = Signal()
    fileChanged = Signal()
    isPlayingChanged = Signal()
    decodingStatusChanged = Signal(QSoundEffect.Status, str)

    def __init__(self, parent=None):
        super().__init__(parent)
        self._sound_effect = QSoundEffect()
        self._sound_effect.playingChanged.connect(self.isPlayingChanged.emit)  #
        self._sound_effect.statusChanged.connect(self.reportStatus)

    def reportStatus(self):
        if self._sound_effect.status() == QSoundEffect.Status.Error:
            self.decodingStatusChanged.emit(
                QSoundEffect.Status.Error,
                f"Error decoding file: {self._sound_effect.source().path()}",
            )
        else:
            self.decodingStatusChanged.emit(self._sound_effect.status(), "")

    @Slot(result=None)
    def play(self):
        self._sound_effect.play()

    def volume(self):
        return self._sound_effect.volume()

    def setVolume(self, value):
        self._sound_effect.setVolume(value)
        self.volumeChanged.emit()

    def file(self):
        return self._sound_effect.source()

    def setFile(self, value: QUrl):
        if self._sound_effect.source() == value or value.isEmpty():
            return

        if "__compiled__" in globals():
            self._sound_effect.setSource(f"qrc:/{value.toString()}")
        else:
            self._sound_effect.setSource(f"file:{project_root / value.toString()}")
        self.fileChanged.emit()

    def isPlaying(self):
        return self._sound_effect.isPlaying()

    volume = Property(float, volume, setVolume, notify=volumeChanged)
    file = Property(QUrl, file, setFile, notify=fileChanged)
    isPlaying = Property(bool, isPlaying, notify=isPlayingChanged)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for Q3DExtras'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import Property, QObject, QTimer, Signal
from PySide6.QtGui import QMatrix4x4, QQuaternion, QVector3D
from PySide6.Qt3DCore import Qt3DCore
from PySide6.Qt3DExtras import Qt3DExtras


class OrbitTransformController(QObject):
    def __init__(self, parent):
        super().__init__(parent)
        self._target = None
        self._matrix = QMatrix4x4()
        self._radius = 1
        self._angle = 0

    def setTarget(self, t):
        self._target = t

    def getTarget(self):
        return self._target

    def setRadius(self, radius):
        if self._radius != radius:
            self._radius = radius
            self.updateMatrix()
            self.radiusChanged.emit()

    def getRadius(self):
        return self._radius

    def setAngle(self, angle):
        if self._angle != angle:
            self._angle = angle
            self.updateMatrix()
            self.angleChanged.emit()

    def getAngle(self):
        return self._angle

    def updateMatrix(self):
        self._matrix.setToIdentity()
        self._matrix.rotate(self._angle, QVector3D(0, 1, 0))
        self._matrix.translate(self._radius, 0, 0)
        if self._target is not None:
            self._target.setMatrix(self._matrix)

    angleChanged = Signal()
    radiusChanged = Signal()
    angle = Property(float, getAngle, setAngle, notify=angleChanged)
    radius = Property(float, getRadius, setRadius, notify=radiusChanged)


class Window(Qt3DExtras.Qt3DWindow):
    def __init__(self):
        super().__init__()

        # Camera
        self.camera().lens().setPerspectiveProjection(45, 16 / 9, 0.1, 1000)
        self.camera().setPosition(QVector3D(0, 0, 40))
        self.camera().setViewCenter(QVector3D(0, 0, 0))

        # For camera controls
        self.createScene()
        self.camController = Qt3DExtras.QOrbitCameraController(self.rootEntity)
        self.camController.setLinearSpeed(50)
        self.camController.setLookSpeed(180)
        self.camController.setCamera(self.camera())

        self.setRootEntity(self.rootEntity)

    def createScene(self):
        # Root entity
        self.rootEntity = Qt3DCore.QEntity()

        # Material
        self.material = Qt3DExtras.QPhongMaterial(self.rootEntity)

        # Torus
        self.torusEntity = Qt3DCore.QEntity(self.rootEntity)
        self.torusMesh = Qt3DExtras.QTorusMesh()
        self.torusMesh.setRadius(5)
        self.torusMesh.setMinorRadius(1)
        self.torusMesh.setRings(100)
        self.torusMesh.setSlices(20)

        self.torusTransform = Qt3DCore.QTransform()
        self.torusTransform.setScale3D(QVector3D(1.5, 1, 0.5))
        self.torusTransform.setRotation(QQuaternion.fromAxisAndAngle(QVector3D(1, 0, 0), 45))

        self.torusEntity.addComponent(self.torusMesh)
        self.torusEntity.addComponent(self.torusTransform)
        self.torusEntity.addComponent(self.material)

        # Sphere
        self.sphereEntity = Qt3DCore.QEntity(self.rootEntity)
        self.sphereMesh = Qt3DExtras.QSphereMesh()
        self.sphereMesh.setRadius(3)

        self.sphereTransform = Qt3DCore.QTransform()
        self.controller = OrbitTransformController(self.sphereTransform)
        self.controller.setTarget(self.sphereTransform)
        self.controller.setRadius(20)

        self.sphereEntity.addComponent(self.sphereMesh)
        self.sphereEntity.addComponent(self.sphereTransform)
        self.sphereEntity.addComponent(self.material)


class Qt3DExtrasTestCase(UsesQApplication):
    '''Tests related to Q3DExtras'''

    def test3DExtras(self):

        window = Window()
        window.show()
        while not window.isExposed():
            self.app.processEvents()
        QTimer.singleShot(2000, self.app.quit)
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 Charts example: Simple memory usage viewer"""

import os
import sys
from PySide6.QtCore import QProcess
from PySide6.QtWidgets import QApplication, QMainWindow
from PySide6.QtCharts import QChart, QChartView, QPieSeries


def run_process(command, arguments):
    process = QProcess()
    process.start(command, arguments)
    process.waitForFinished()
    std_output = process.readAllStandardOutput().data().decode('utf-8')
    return std_output.split('\n')


def get_memory_usage():
    result = []
    if sys.platform == 'win32':
        # Windows: Obtain memory usage in KB from 'tasklist'
        for line in run_process('tasklist', [])[3:]:
            if len(line) >= 74:
                command = line[0:23].strip()
                if command.endswith('.exe'):
                    command = command[0:len(command) - 4]
                memory_usage = float(line[64:74].strip().replace(',', '').replace('.', ''))
                legend = ''
                if memory_usage > 10240:
                    mb = memory_usage / 1024
                    legend = f'{command} {mb}M'
                else:
                    legend = f'{command} {memory_usage}K'
                result.append([legend, memory_usage])
    else:
        # Unix: Obtain memory usage percentage from 'ps'
        ps_options = ['-e', 'v']
        memory_column = 8
        command_column = 9
        if sys.platform == 'darwin':
            ps_options = ['-e', '-v']
            memory_column = 11
            command_column = 12
        for line in run_process('ps', ps_options):
            tokens = line.split(None)
            if len(tokens) > command_column and "PID" not in tokens:  # Percentage and command
                command = tokens[command_column]
                if not command.startswith('['):
                    command = os.path.basename(command)
                    memory_usage = round(float(tokens[memory_column].replace(',', '.')))
                    legend = f'{command} {memory_usage}%'
                    result.append([legend, memory_usage])

    result.sort(key=lambda x: x[1], reverse=True)
    return result


class MainWindow(QMainWindow):

    def __init__(self):
        super().__init__()

        self.setWindowTitle('Memory Usage')

        memory_usage = get_memory_usage()
        if len(memory_usage) > 5:
            memory_usage = memory_usage[0:4]

        self.series = QPieSeries()
        for item in memory_usage:
            self.series.append(item[0], item[1])

        chart_slice = self.series.slices()[0]
        chart_slice.setExploded()
        chart_slice.setLabelVisible()
        self.chart = QChart()
        self.chart.addSeries(self.series)
        self._chart_view = QChartView(self.chart)
        self.setCentralWidget(self._chart_view)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    main_win = MainWindow()
    available_geometry = main_win.screen().availableGeometry()
    size = available_geometry.height() * 3 / 4
    main_win.resize(size, size)
    main_win.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""
feature_with_uic_test.py
------------------------

Check if feature switching works with a normal UIC file.
This crashed due to callbacks into QApplication.

PYSIDE-1626: Switch early in `BindingManager::getOverride`.
"""

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import QCoreApplication, qVersion
from PySide6.QtWidgets import QMainWindow

# PYSIDE-535: We cannot use __feature__ in PyPy, yet
try:
    from __feature__ import snake_case  # noqa

    from feature_with_uic.window import Ui_MainWindow
    have_feature = True
except ImportError:
    Ui_MainWindow = object
    have_feature = False


class MainWindow(QMainWindow, Ui_MainWindow):

    def __init__(self) -> None:
        super().__init__()
        self.setupUi(self)


@unittest.skipIf(hasattr(sys, "pypy_version_info"),
                 "__feature__ cannot yet be used with PyPy")
class FeatureTest(UsesQApplication):

    def testFeaturesWorkWithUIC(self):
        window = MainWindow()
        window.set_window_title(qVersion())
        window.show()
        while not window.window_handle().is_exposed():
            QCoreApplication.process_events()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal
o = QObject()


class MyObject(QObject):
    s = Signal(int)


class CheckSignalType(unittest.TestCase):
    def testSignal(self):
        self.assertTrue(isinstance(QObject.destroyed, Signal))
        self.assertEqual(type(QObject.destroyed), Signal)

        self.assertEqual(type(o.destroyed).__name__, "SignalInstance")
        self.assertNotEqual(type(o.destroyed), Signal)

        self.assertTrue(isinstance(o.destroyed, Signal))
        self.assertTrue(isinstance(MyObject.s, Signal))
        self.assertFalse(isinstance(int, Signal))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal


class Emitter(QObject):
    s1 = Signal()
    s2 = Signal()
    s3 = Signal()
    s4 = Signal()
    s5 = Signal()
    s6 = Signal()
    s7 = Signal()
    s8 = Signal()
    s9 = Signal()
    s10 = Signal()
    s11 = Signal()
    s12 = Signal()
    s13 = Signal()
    s14 = Signal()


class SignalNumberLimitTest(unittest.TestCase):
    def myCb(self):
        self._count += 1

    def testBug(self):
        e = Emitter()
        e.s1.connect(self.myCb)
        e.s2.connect(self.myCb)
        e.s3.connect(self.myCb)
        e.s4.connect(self.myCb)
        e.s5.connect(self.myCb)
        e.s6.connect(self.myCb)
        e.s7.connect(self.myCb)
        e.s8.connect(self.myCb)
        e.s9.connect(self.myCb)
        e.s10.connect(self.myCb)
        e.s11.connect(self.myCb)
        e.s12.connect(self.myCb)
        e.s13.connect(self.myCb)
        e.s14.connect(self.myCb)

        self._count = 0
        e.s1.emit()
        e.s2.emit()
        e.s3.emit()
        e.s4.emit()
        e.s5.emit()
        e.s6.emit()
        e.s7.emit()
        e.s8.emit()
        e.s9.emit()
        e.s10.emit()
        e.s11.emit()
        e.s12.emit()
        e.s13.emit()
        e.s14.emit()
        self.assertEqual(self._count, 14)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for virtual methods.'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point, Str, StrList, VirtualDaughter, VirtualMethods

import warnings


class ExtendedVirtualMethods(VirtualMethods):
    def __init__(self):
        VirtualMethods.__init__(self)
        self.virtual_method0_called = False

    def virtualMethod0(self, pt, val, cpx, b):
        self.virtual_method0_called = True
        return VirtualMethods.virtualMethod0(self, pt, val, cpx, b) * -1.0

    def strListToStdList(self, arg):
        warnings.simplefilter('error')
        # returning wrong type for test purposes.
        return True

    def recursionOnModifiedVirtual(self, arg):
        # check if recursion is caused by injected code that calls C++.
        return VirtualMethods.recursionOnModifiedVirtual(self, arg) + 10


class ExtendedVirtualDaughter(VirtualDaughter):
    def __init__(self, name):
        VirtualDaughter.__init__(self, name)
        self.grand_daughter_name_called = False

    def name(self):
        self.grand_daughter_name_called = True
        return VirtualDaughter.name(self).prepend('Extended')


class ExtendedExtendedVirtualDaughter(ExtendedVirtualDaughter):
    def __init__(self, name):
        ExtendedVirtualDaughter.__init__(self, name)
        self.grand_grand_daughter_name_called = False

    def name(self):
        self.grand_grand_daughter_name_called = True
        return ExtendedVirtualDaughter.name(self).prepend('Extended')


class VirtualMethodsTest(unittest.TestCase):
    '''Test case for virtual methods'''

    def setUp(self):
        self.prefix_from_codeinjection = Str('Pimped')

    def tearDown(self):
        del self.prefix_from_codeinjection
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testReimplementedVirtualMethod0(self):
        '''Test Python override of a virtual method with various different parameters
           is correctly called from C++.'''
        vm = VirtualMethods()
        evm = ExtendedVirtualMethods()
        pt = Point(1.1, 2.2)
        val = 4
        cpx = complex(3.3, 4.4)
        b = True
        result0 = vm.callVirtualMethod0(pt, val, cpx, b)
        result1 = evm.callVirtualMethod0(pt, val, cpx, b)
        self.assertEqual(result0 * -1.0, result1)

    def testRecursionOnModifiedVirtual(self):
        evm = ExtendedVirtualMethods()
        self.assertEqual(evm.recursionOnModifiedVirtual(''), 10)
        self.assertEqual(evm.callRecursionOnModifiedVirtual(''), 10)

    def testReimplementedVirtualMethodInheritedFromGrandParent(self):
        '''Test Python override of a virtual method inherited from a grand parent.'''
        original_name = 'Foo'
        evd = ExtendedVirtualDaughter(original_name)

        self.assertEqual(VirtualDaughter.name(evd), original_name)
        self.assertEqual(VirtualMethods.name(evd), original_name)
        self.assertFalse(evd.grand_daughter_name_called)

        name = evd.callName()
        self.assertTrue(evd.grand_daughter_name_called)
        self.assertEqual(evd.name().prepend(self.prefix_from_codeinjection), name)

    def testReimplementedVirtualMethodInheritedFromGrandGrandParent(self):
        '''Test Python override of a virtual method inherited from a grand grand parent.'''
        original_name = 'Foo'
        eevd = ExtendedExtendedVirtualDaughter(original_name)

        self.assertEqual(VirtualDaughter.name(eevd), original_name)
        self.assertEqual(VirtualMethods.name(eevd), original_name)
        self.assertFalse(eevd.grand_daughter_name_called)
        self.assertFalse(eevd.grand_grand_daughter_name_called)

        name = eevd.callName()
        self.assertTrue(eevd.grand_daughter_name_called)
        self.assertTrue(eevd.grand_grand_daughter_name_called)
        self.assertEqual(eevd.name().prepend(self.prefix_from_codeinjection), name)

    def testStringView(self):
        virtual_methods = VirtualMethods()
        self.assertEqual(virtual_methods.stringViewLength('bla'), 3)


class PrettyErrorMessageTest(unittest.TestCase):
    def testIt(self):
        obj = ExtendedVirtualMethods()
        self.assertRaises(RuntimeWarning, obj.callStrListToStdList, StrList())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QGraphicsScene


class TestBug679(unittest.TestCase):
    '''QGraphicsScene::clear() is missing'''
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testIt(self):
        app = QApplication([])  # noqa: F841

        scene = QGraphicsScene()
        hello = scene.addText("Hello")
        base_ref_count = sys.getrefcount(hello)
        scene.addText("World")

        scene.clear()
        self.assertEqual(sys.getrefcount(hello), base_ref_count - 1)
        self.assertEqual(len(scene.items()), 0)
        self.assertRaises(RuntimeError, hello.isVisible)  # the C++ object was deleted


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6 import *  # noqa: F401,F403
from PySide6.support.signature import get_signature
for modname, mod in sys.modules.items():
    if modname.startswith("PySide6."):
        print("importing", modname)
        exec("import " + modname)

# To help linters
import PySide6.QtCore
import PySide6.QtGui
import PySide6.QtWidgets

# This test tests the existence and callability of the newly existing functions,
# after the inheritance was made complete in the course of PYSIDE-331.

new_functions = """
    PySide6.QtCore.QAbstractItemModel().parent()
    PySide6.QtCore.QAbstractListModel().parent()
    PySide6.QtCore.QAbstractTableModel().parent()
    PySide6.QtCore.QFile().resize(qint64)
    # prevent a message "QMutex: destroying locked mutex"
    m = PySide6.QtCore.QMutex(); m.tryLock(); m.unlock()
    PySide6.QtCore.QSortFilterProxyModel().parent()
    PySide6.QtCore.QTemporaryFile(tfarg).open(openMode)
"""

new_functions += """
    PySide6.QtGui.QStandardItemModel().insertColumn(_int,qModelIndex)
    PySide6.QtGui.QStandardItemModel().parent()
    # PySide6.QtGui.QTextList(qTextDocument).setFormat(qTextFormat) # Segmentation fault: 11
    # PySide6.QtGui.QTextTable(qTextDocument).setFormat(qTextFormat) # Segmentation fault: 11
""" if "PySide6.QtGui" in sys.modules else ""

new_functions += """
    PySide6.QtWidgets.QAbstractItemView().update()
    PySide6.QtWidgets.QApplication.palette()
    PySide6.QtWidgets.QApplication.setFont(qFont)
    PySide6.QtWidgets.QApplication.setPalette(qPalette)
    PySide6.QtWidgets.QBoxLayout(direction).addWidget(qWidget)
    PySide6.QtWidgets.QColorDialog().open()
    PySide6.QtWidgets.QFileDialog().open()
    PySide6.QtWidgets.QFileSystemModel().index(_int,_int,qModelIndex)
    PySide6.QtWidgets.QFileSystemModel().parent()
    PySide6.QtWidgets.QFontDialog().open()
    PySide6.QtWidgets.QGestureEvent([]).accept()
    PySide6.QtWidgets.QGestureEvent([]).ignore()
    PySide6.QtWidgets.QGestureEvent([]).isAccepted()
    PySide6.QtWidgets.QGestureEvent([]).setAccepted(_bool)
    # QPaintDevice: NotImplementedError
    # PySide6.QtWidgets.QGraphicsView().render(qPaintDevice,qPoint,qRegion,renderFlags)
    PySide6.QtWidgets.QGridLayout().addWidget(qWidget)
    PySide6.QtWidgets.QInputDialog().open()
    PySide6.QtWidgets.QLineEdit().addAction(qAction)
    PySide6.QtWidgets.QMessageBox().open()
    PySide6.QtWidgets.QPlainTextEdit().find(findStr)
    PySide6.QtWidgets.QProgressDialog().open()
    PySide6.QtWidgets.QStackedLayout().widget()
    # PySide6.QtWidgets.QStylePainter().begin(qPaintDevice) # QPaintDevice: NotImplementedError
    PySide6.QtWidgets.QTextEdit().find(findStr)
    PySide6.QtWidgets.QWidget.find(quintptr)
""" if "PySide6.QtWidgets" in sys.modules else ""

new_functions += """
    # PySide6.QtPrintSupport.QPageSetupDialog().open() # Segmentation fault: 11
    # PySide6.QtPrintSupport.QPrintDialog().open() # opens the dialog, but works
    PySide6.QtPrintSupport.QPrintDialog().printer()
    # note: this prints something, but really shouldn't ;-)
    PySide6.QtPrintSupport.QPrintPreviewDialog().open()
""" if "PySide6.QtPrintSupport" in sys.modules else ""

new_functions += """
    PySide6.QtHelp.QHelpContentModel().parent()
    # returns NULL without setting an error
    # PySide6.QtHelp.QHelpIndexModel().createIndex(_int,_int,quintptr)
    # returns NULL without setting an error
    # PySide6.QtHelp.QHelpIndexModel().createIndex(_int,_int,object())
""" if "PySide6.QtHelp" in sys.modules else ""

new_functions += """
    PySide6.QtQuick.QQuickPaintedItem().update()
""" if "PySide6.QtQuick" in sys.modules else ""


class MainTest(unittest.TestCase):

    def testNewInheriedFunctionsExist(self):
        """
        Run all new method signarures
        """
        for app in "QtWidgets.QApplication", "QtGui.QGuiApplication", "QtCore.QCoreApplication":
            try:
                exec(f"qApp = PySide6.{app}([]) or PySide6.{app}.instance()")
                break
            except AttributeError:
                continue
        _bool = True  # noqa: F841,F405
        _int = 42  # noqa: F841,F405
        qint64 = 42  # noqa: F841,F405
        tfarg = os.path.join(PySide6.QtCore.QDir.tempPath(), "XXXXXX.tmp")  # noqa: F841,F405
        findStr = 'bla'  # noqa: F841,F405
        orientation = PySide6.QtCore.Qt.Orientations()  # noqa: F841,F405
        openMode = PySide6.QtCore.QIODevice.OpenMode(PySide6.QtCore.QIODevice.OpenModeFlag.ReadOnly)  # noqa: F841,F405,E501
        qModelIndex = PySide6.QtCore.QModelIndex()  # noqa: F841,F405
        transformationMode = PySide6.QtCore.Qt.TransformationMode()  # noqa: F841,F405
        qObject = PySide6.QtCore.QObject()  # noqa: F841,F405
        qPoint = PySide6.QtCore.QPoint()  # noqa: F841,F405
        try:
            PySide6.QtGui  # noqa: F405
            #qPaintDevice = PySide6.QtGui.QPaintDevice()  # NotImplementedError
            qTextDocument = PySide6.QtGui.QTextDocument()  # noqa: F841,F405
            qTextFormat = PySide6.QtGui.QTextFormat()  # noqa: F841,F405
            quintptr = 42  # noqa: F841,F405
            qFont = PySide6.QtGui.QFont()  # noqa: F841,F405
            qPalette = PySide6.QtGui.QPalette()  # noqa: F841,F405
        except AttributeError:
            pass
        try:
            PySide6.QtWidgets  # noqa: F405
            direction = PySide6.QtWidgets.QBoxLayout.Direction()  # noqa: F841,F405
            qWidget = PySide6.QtWidgets.QWidget()  # noqa: F841,F405
            qStyleOptionFrame = PySide6.QtWidgets.QStyleOptionFrame()  # noqa: F841,F405
            qAction = PySide6.QtGui.QAction(qObject)  # noqa: F841,F405
            renderFlags = PySide6.QtWidgets.QWidget.RenderFlags  # noqa: F841,F405
        except AttributeError:
            pass

        for func in new_functions.splitlines():
            func = func.strip()
            if func.startswith("#"):
                # this is a crashing or otherwise untestable function
                print(func)
                continue
            try:
                exec(func)
            except NotImplementedError:
                print(func, "# raises NotImplementedError")
            else:
                print(func)

    def testQAppSignatures(self):
        """
        Verify that qApp.palette owns three signatures, especially
        palette() without argument.
        """
        try:
            qApp = (  # noqa: F841
                PySide6.QtWidgets.QApplication.instance() or PySide6.QtWidgets.QApplication([])  # noqa: F405,E501
            )
        except AttributeError:
            unittest.TestCase().skipTest("this test makes only sense if QtWidgets is available.")

        sigs = get_signature(PySide6.QtWidgets.QApplication.palette)  # noqa: F405
        self.assertEqual(len(sigs), 3)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QRegularExpression, Property, Slot
from PySide6.QtGui import QRegularExpressionValidator
from PySide6.QtWidgets import (QComboBox, QDialog, QDialogButtonBox,
                               QFormLayout, QGroupBox, QHBoxLayout,
                               QLineEdit, QListWidget,
                               QListWidgetItem, QPushButton, QVBoxLayout,
                               )


DEFAULT_TYPES = ['int', 'str', 'PySide6.QtCore.QPoint', 'PySide6.QtCore.QRect',
                 'PySide6.QtCore.QSize', 'PySide6.QtGui.QColor']


FUNCTION_PATTERN = r'^\w+\([\w ,]*\)$'


class ValidatingInputDialog(QDialog):
    """A dialog for text input with a regular expression validation."""
    def __init__(self, label, pattern, parent=None):
        super().__init__(parent)
        layout = QVBoxLayout(self)

        self._form_layout = QFormLayout()
        self._lineedit = QLineEdit()
        self._lineedit.setClearButtonEnabled(True)
        re = QRegularExpression(pattern)
        assert re.isValid()
        self._validator = QRegularExpressionValidator(re, self)
        self._lineedit.setValidator(self._validator)
        self._form_layout.addRow(label, self._lineedit)
        layout.addLayout(self._form_layout)

        bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok
                              | QDialogButtonBox.StandardButton.Cancel)
        layout.addWidget(bb)
        bb.rejected.connect(self.reject)
        bb.accepted.connect(self.accept)

    @Property(str)
    def text(self):
        return self._lineedit.text()

    @text.setter
    def text(self, t):
        self._lineedit.setText(t)

    @Property(str)
    def placeholder_text(self):
        return self._lineedit.placeholderText()

    @placeholder_text.setter
    def placeholder_text(self, t):
        self._lineedit.setPlaceholderText(t)

    @Property(int)
    def cursor_position(self):
        return self._lineedit.cursorPosition()

    @cursor_position.setter
    def cursor_position(self, p):
        self._lineedit.setCursorPosition(p)

    def is_valid(self):
        return self.text

    def accept(self):
        if self.is_valid():
            super(ValidatingInputDialog, self).accept()


class FunctionSignatureDialog(ValidatingInputDialog):
    """A dialog for input of function signatures."""
    def __init__(self, name, parent=None):
        super().__init__(name, FUNCTION_PATTERN, parent)
        self.text = '()'
        self.cursor_position = 0


class PropertyDialog(ValidatingInputDialog):
    """A dialog for input of a property name and type."""
    def __init__(self, parent=None):
        super().__init__('&Name:', r'^\w+$', parent)
        self.setWindowTitle('Add a Property')
        self._type_combo = QComboBox()
        self._type_combo.addItems(DEFAULT_TYPES)
        self._form_layout.insertRow(0, '&Type:', self._type_combo)

    def property_type(self):
        return self._type_combo.currentText()


class ListChooser(QGroupBox):
    """A widget for editing a list of strings with a customization point
       for creating the strings."""
    def __init__(self, title, parent=None):
        super().__init__(title, parent)
        main_layout = QHBoxLayout(self)
        self._list = QListWidget(self)
        self._list.currentItemChanged.connect(self._current_item_changed)
        main_layout.addWidget(self._list)

        vbox_layout = QVBoxLayout()
        main_layout.addLayout(vbox_layout)
        self._addButton = QPushButton("Add...")
        vbox_layout.addWidget(self._addButton)
        self._addButton.clicked.connect(self._add)
        self._removeButton = QPushButton("Remove")
        self._removeButton.setEnabled(False)
        self._removeButton.clicked.connect(self._remove_current)
        vbox_layout.addWidget(self._removeButton)
        vbox_layout.addStretch()

    @Property(list)
    def items(self):
        result = []
        for i in range(self._list.count()):
            result.append(self._list.item(i).text())
        return result

    @items.setter
    def items(self, item_list):
        self._list.clear()
        for i in item_list:
            self._list.append(i)

    @Slot(QListWidgetItem, QListWidgetItem)
    def _current_item_changed(self, current, previous):
        self._removeButton.setEnabled(current is not None)

    @Slot()
    def _add(self):
        if new_item := self._create_new_item():
            self._list.addItem(new_item)

    def _create_new_item(self):
        """Overwrite to return a new item."""
        return 'new_item'

    @Slot()
    def _remove_current(self):
        row = self._list.row(self._list.currentItem())
        if row >= 0:
            self._list.takeItem(row)


class SignalChooser(ListChooser):
    """A widget for editing a list of signal function signatures."""
    def __init__(self, parent=None):
        super().__init__('Signals', parent)

    def _create_new_item(self):
        dialog = FunctionSignatureDialog('&Signal signature:', self)
        dialog.setWindowTitle('Enter Signal')
        if dialog.exec() != QDialog.Accepted:
            return ''
        return dialog.text


class PropertyChooser(ListChooser):
    """A widget for editing a list of properties as a string of 'type name'."""
    def __init__(self, parent=None):
        super().__init__('Properties', parent)

    def _create_new_item(self):
        dialog = PropertyDialog(self)
        if dialog.exec() != QDialog.Accepted:
            return ''
        name = dialog.text
        property_type = dialog.property_type()
        return f'{property_type} {name}'



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Selected Point Configuration Example from Qt 6.5"""
from PySide6.QtCore import QPointF, Slot
from PySide6.QtGui import QColor, QIcon, QPainter
from PySide6.QtWidgets import QMainWindow, QLineEdit, QLabel, QComboBox
from PySide6.QtWidgets import QCheckBox, QWidget, QGridLayout, QHBoxLayout
from PySide6.QtCharts import QLineSeries, QXYSeries, QChart, QChartView


PointConfig = QXYSeries.PointConfiguration


class ChartWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setWindowTitle("Chart")
        self._series = QLineSeries(self)
        self._series.setName("Customized series")
        self._series.setPointsVisible(True)
        self._series.append([QPointF(0, 7), QPointF(2, 4),
                             QPointF(3, 5), QPointF(7, 4),
                             QPointF(10, 5), QPointF(11, 1),
                             QPointF(13, 3), QPointF(17, 6),
                             QPointF(18, 3), QPointF(20, 2)])

        selected_point_index_label = QLabel("Selected Point: ")
        self._selected_point_index_lineedit = QLineEdit()
        self._selected_point_index_lineedit.setReadOnly(True)
        self._selected_point_index_lineedit.setStyleSheet(
            "background-color: rgba(0, 0, 0, 0); border: 0px")

        color_label = QLabel("Color: ")
        self._color_combobox = QComboBox()
        color_strings = ["red", "orange", "yellow", "green", "blue",
                         "indigo", "violet", "black"]
        for color_str in color_strings:
            self._color_combobox.addItem(QIcon(), color_str, QColor(color_str))

        size_label = QLabel("Size: ")
        self._size_combobox = QComboBox()
        for size in [2, 3, 4, 6, 8, 10, 12, 15]:
            self._size_combobox.addItem(QIcon(), str(size), size)

        label_visibility_label = QLabel("Label Visibility: ")
        self._label_visibility_checkbox = QCheckBox()

        custom_label_label = QLabel("Custom Label: ")
        self._custom_label_lineedit = QLineEdit()

        self._series.clicked.connect(self._select_point)
        self._color_combobox.activated.connect(self._set_color)
        self._size_combobox.activated.connect(self._set_size)
        label_vis_checkbox = self._label_visibility_checkbox
        label_vis_checkbox.clicked.connect(self._set_label_visibility)
        clabel_lineedit = self._custom_label_lineedit
        clabel_lineedit.editingFinished.connect(self._set_custom_label)

        self._chart = QChart()
        self._chart.addSeries(self._series)
        self._chart.createDefaultAxes()

        chart_view = QChartView(self._chart)
        chart_view.setRenderHint(QPainter.RenderHint.Antialiasing)

        control_widget = QWidget(self)
        control_layout = QGridLayout(control_widget)
        control_layout.setColumnStretch(1, 1)

        control_layout.addWidget(selected_point_index_label, 0, 0)
        control_layout.addWidget(self._selected_point_index_lineedit, 0, 1)

        control_layout.addWidget(color_label, 1, 0)
        control_layout.addWidget(self._color_combobox, 1, 1)

        control_layout.addWidget(size_label, 2, 0)
        control_layout.addWidget(self._size_combobox, 2, 1)

        control_layout.addWidget(label_visibility_label, 3, 0)
        control_layout.addWidget(self._label_visibility_checkbox, 3, 1, 1, 2)

        control_layout.addWidget(custom_label_label, 4, 0)
        control_layout.addWidget(self._custom_label_lineedit, 4, 1)

        main_widget = QWidget(self)
        main_layout = QHBoxLayout(main_widget)
        main_layout.addWidget(chart_view)
        main_layout.setStretch(0, 1)
        main_layout.addWidget(control_widget)
        self.setCentralWidget(main_widget)

        self._select_point(4)

    @Slot(QPointF)
    def _select_point(self, point: QPointF | int):
        try:
            index = (self._series.points().index(point.toPoint()) if
                     isinstance(point, QPointF) else point)
        except ValueError:
            # Do nothing if the place that was clicked on wasn't a point.
            return

        self._series.deselectAllPoints()
        self._series.selectPoint(index)
        self._selectedPointIndex = index
        self._selectedPointConfig = self._series.pointConfiguration(index)
        selected_point = self._series.at(index)
        selected_index_lineedit = self._selected_point_index_lineedit
        selected_index_lineedit.setText("(" + str(selected_point.x()) + ", "
                                        + str(selected_point.y()) + ")")
        config = self._series.pointConfiguration(index)

        color = config.get(PointConfig.Color) or self._series.color()
        size = config.get(PointConfig.Size) or self._series.markerSize()
        labelVisibility = (config.get(PointConfig.LabelVisibility)
                           or self._series.pointLabelsVisible())
        customLabel = config.get(PointConfig.LabelFormat) or ""

        combobox_value_list = [
            (self._color_combobox, color.name(), color),
            (self._size_combobox, str(size), size)
        ]
        for box, value_str, value in combobox_value_list:
            if box.findData(value) < 0:
                box.addItem(value_str, value)
            box.setCurrentIndex(box.findData(value))

        self._label_visibility_checkbox.setChecked(labelVisibility)
        self._custom_label_lineedit.setText(customLabel)

    @Slot(int)
    def _set_color(self, index: int):
        spc = self._selectedPointConfig
        spc[PointConfig.Color] = self._color_combobox.currentData()
        self._series.setPointConfiguration(self._selectedPointIndex, spc)

    @Slot(int)
    def _set_size(self, index: int):
        spc = self._selectedPointConfig
        spc[PointConfig.Size] = self._size_combobox.currentData()
        self._series.setPointConfiguration(self._selectedPointIndex, spc)

    @Slot(bool)
    def _set_label_visibility(self, checked: bool):
        spc = self._selectedPointConfig
        spc[PointConfig.LabelVisibility] = checked
        self._series.setPointConfiguration(self._selectedPointIndex, spc)

    @Slot()
    def _set_custom_label(self):
        spc = self._selectedPointConfig
        spc[PointConfig.LabelFormat] = self._custom_label_lineedit.text()
        self._series.setPointConfiguration(self._selectedPointIndex, spc)



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt Simple RHI Widget Example example from Qt v6.x"""

import sys

from PySide6.QtWidgets import QApplication, QVBoxLayout, QWidget

from examplewidget import ExampleRhiWidget
import rc_simplerhiwidget  # noqa F:401


class Widget(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)
        layout = QVBoxLayout(self)
        self._rhi_widget = ExampleRhiWidget(self)
        layout.addWidget(self._rhi_widget)

    def closeEvent(self, e):
        self._rhi_widget.releaseResources()
        e.accept()


if __name__ == "__main__":
    app = QApplication(sys.argv)

    w = Widget()
    w.resize(1280, 720)
    w.show()
    exit_code = app.exec()
    del w
    sys.exit(exit_code)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for a class with a private destructor.'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from shiboken6 import Shiboken
from sample import PrivateDtor


class PrivateDtorTest(unittest.TestCase):
    '''Test case for PrivateDtor class'''

    def assertRaises(self, *args, **kwds):
        if not hasattr(sys, "pypy_version_info"):
            # PYSIDE-535: PyPy complains "Fatal RPython error: NotImplementedError"
            return super().assertRaises(*args, **kwds)

    def testPrivateDtorInstanciation(self):
        '''Test if instanciation of class with a private destructor raises an exception.'''
        self.assertRaises(TypeError, PrivateDtor)

    def testPrivateDtorInheritance(self):
        '''Test if inheriting from PrivateDtor raises an exception.'''
        def inherit():
            class Foo(PrivateDtor):
                pass
        self.assertRaises(TypeError, inherit)

    def testPrivateDtorInstanceMethod(self):
        '''Test if PrivateDtor.instance() method return the proper singleton.'''
        pd1 = PrivateDtor.instance()
        calls = pd1.instanceCalls()
        self.assertEqual(type(pd1), PrivateDtor)
        pd2 = PrivateDtor.instance()
        self.assertEqual(pd2, pd1)
        self.assertEqual(pd2.instanceCalls(), calls + 1)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testPrivateDtorRefCounting(self):
        '''Test refcounting of the singleton returned by PrivateDtor.instance().'''
        pd1 = PrivateDtor.instance()
        calls = pd1.instanceCalls()
        refcnt = sys.getrefcount(pd1)
        pd2 = PrivateDtor.instance()
        self.assertEqual(pd2.instanceCalls(), calls + 1)
        self.assertEqual(sys.getrefcount(pd2), sys.getrefcount(pd1))
        self.assertEqual(sys.getrefcount(pd2), refcnt + 1)
        del pd1
        self.assertEqual(sys.getrefcount(pd2), refcnt)
        del pd2
        gc.collect()
        pd3 = PrivateDtor.instance()
        self.assertEqual(type(pd3), PrivateDtor)
        self.assertEqual(pd3.instanceCalls(), calls + 2)
        self.assertEqual(sys.getrefcount(pd3), refcnt)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testClassDecref(self):
        # Bug was that class PyTypeObject wasn't decrefed when instance
        # was invalidated

        before = sys.getrefcount(PrivateDtor)

        for i in range(1000):
            obj = PrivateDtor.instance()
            Shiboken.invalidate(obj)

        after = sys.getrefcount(PrivateDtor)

        self.assertLess(abs(before - after), 5)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal


class Sender(QObject):
    """Sender class used in this test."""

    foo = Signal()
    foo_int = Signal(int)
    foo_int_int_string = Signal(int, int, str)
    foo_int_qobject = Signal(int, QObject)

    def __init__(self, parent=None):
        QObject.__init__(self, parent)


class ShortCircuitSignals(unittest.TestCase):
    def setUp(self):
        self.called = False

    def tearDown(self):
        try:
            del self.args
        except:  # noqa: E722
            pass
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def callback(self, *args):
        if tuple(self.args) == args:
            self.called = True

    def testNoArgs(self):
        """Short circuit signal without arguments"""
        obj1 = Sender()
        obj1.foo.connect(self.callback)
        self.args = tuple()
        obj1.foo.emit(*self.args)
        self.assertTrue(self.called)

    def testWithArgs(self):
        """Short circuit signal with integer arguments"""
        obj1 = Sender()

        obj1.foo_int.connect(self.callback)
        self.args = (42,)
        obj1.foo_int.emit(*self.args)

        self.assertTrue(self.called)

    def testMultipleArgs(self):
        """Short circuit signal with multiple arguments"""
        obj1 = Sender()

        obj1.foo_int_int_string.connect(self.callback)
        self.args = (42, 33, 'char')
        obj1.foo_int_int_string.emit(*self.args)

        self.assertTrue(self.called)

    def testComplexArgs(self):
        """Short circuit signal with complex arguments"""
        obj1 = Sender()

        obj1.foo_int_qobject.connect(self.callback)
        self.args = (42, obj1)

        obj1.foo_int_qobject.emit(*self.args)
        self.assertTrue(self.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt Document Viewer demo from Qt v6.x"""

import sys
from argparse import ArgumentParser, RawTextHelpFormatter

from PySide6.QtWidgets import QApplication
from PySide6.QtCore import QCoreApplication

from mainwindow import MainWindow


DESCRIPTION = "A viewer for JSON, PDF and text files"


if __name__ == "__main__":

    app = QApplication([])
    QCoreApplication.setOrganizationName("QtExamples")
    QCoreApplication.setApplicationName("DocumentViewer")
    QCoreApplication.setApplicationVersion("1.0")

    arg_parser = ArgumentParser(description=DESCRIPTION,
                                formatter_class=RawTextHelpFormatter)
    arg_parser.add_argument("file", type=str, nargs="?",
                            help="JSON, PDF or text file to open")
    args = arg_parser.parse_args()
    fileName = args.file

    w = MainWindow()
    w.show()
    if args.file and not w.openFile(args.file):
        sys.exit(-1)

    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QDate'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QDate


class TestQDate (unittest.TestCase):
    def testGetDate(self):
        date = QDate(2009, 22, 9)
        tuple_ = date.getDate()
        self.assertEqual(tuple, tuple_.__class__)
        (y, m, d) = tuple_
        self.assertEqual(date.year(), y)
        self.assertEqual(date.month(), m)
        self.assertEqual(date.day(), d)

    def testGetWeekNumber(self):
        date = QDate(2000, 1, 1)
        tuple_ = date.weekNumber()
        self.assertEqual(tuple, tuple_.__class__)
        (week, yearNumber) = tuple_
        self.assertEqual(week, 52)
        self.assertEqual(yearNumber, 1999)

    def testBooleanCast(self):
        today = QDate.currentDate()
        self.assertTrue(today)
        nodate = QDate()
        self.assertFalse(nodate)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for ...'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType


class DerivedObjectType(ObjectType):
    def isPython(self):
        return True

    def createChild(self, parent):
        return DerivedObjectType(parent)


class ParentTest(unittest.TestCase):

    def testUunsafeParent(self):
        o = DerivedObjectType()
        o.callVirtualCreateChild()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import QObject, Property, Signal

from heartrate_global import simulator


class DeviceInfo(QObject):

    deviceChanged = Signal()

    def __init__(self, device):
        super().__init__()
        self.m_device = device

    def device(self):
        return self.m_device

    def setDevice(self, device):
        self.m_device = device
        self.deviceChanged.emit()

    @Property(str, notify=deviceChanged)
    def deviceName(self):
        if simulator():
            return "Demo BT device"
        return self.m_device.name()

    @Property(str, notify=deviceChanged)
    def deviceAddress(self):
        if simulator():
            return "00:11:22:33:44:55"
        if sys.platform == "Darwin":  # workaround for Core Bluetooth:
            return self.m_device.deviceUuid().toString()
        return self.m_device.address().toString()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import os
from pathlib import Path

from PySide6.QtWidgets import QFileDialog, QDialog
from PySide6.QtCore import QBuffer, QIODeviceBase, Slot, QSharedMemory, QDataStream, qVersion
from PySide6.QtGui import QImage, QPixmap
from ui_dialog import Ui_Dialog


class Dialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)

        v = qVersion()
        name = f"QSharedMemoryExample_v{v}"
        self._shared_memory = QSharedMemory(name)

        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.ui.loadFromFileButton.clicked.connect(self.load_from_file)
        self.ui.loadFromSharedMemoryButton.clicked.connect(self.load_from_memory)
        self.setWindowTitle("SharedMemory Example")

    def ensure_detached(self):
        if self._shared_memory.isAttached():
            self.detach()

    def closeEvent(self, e):
        self.ensure_detached()
        e.accept()

    @Slot()
    def load_from_file(self):
        self.ensure_detached()

        self.ui.label.setText("Select an image file")
        dir = Path(__file__).resolve().parent
        fileName, _ = QFileDialog.getOpenFileName(self, "Choose Image",
                                                  os.fspath(dir),
                                                  "Images (*.png *.jpg)")
        if not fileName:
            return
        image = QImage()
        if not image.load(fileName):
            self.ui.label.setText("Selected file is not an image, please select another.")
            return
        self.ui.label.setPixmap(QPixmap.fromImage(image))

        # load into shared memory
        buffer = QBuffer()
        buffer.open(QIODeviceBase.OpenModeFlag.WriteOnly)
        out = QDataStream(buffer)
        out << image
        buffer.close()
        size = buffer.size()

        if not self._shared_memory.create(size):
            self.ui.label.setText("Unable to create shared memory segment.")
            return

        self._shared_memory.lock()
        _to = memoryview(self._shared_memory.data())
        _from = buffer.data().data()
        _to[0:size] = _from[0:size]
        self._shared_memory.unlock()

    @Slot()
    def load_from_memory(self):
        if not self._shared_memory.isAttached() and not self._shared_memory.attach():
            self.ui.label.setText("Unable to attach to shared memory segment.\n"
                                  "Load an image first.")
            return

        self._shared_memory.lock()
        mv = memoryview(self._shared_memory.constData())
        buffer = QBuffer()
        buffer.setData(mv.tobytes())
        buffer.open(QBuffer.OpenModeFlag.ReadOnly)
        _in = QDataStream(buffer)
        image = QImage()
        _in >> image
        buffer.close()
        self._shared_memory.unlock()
        self._shared_memory.detach()

        self.ui.label.setPixmap(QPixmap.fromImage(image))

    def detach(self):
        if not self._shared_memory.detach():
            self.ui.label.setText(self.tr("Unable to detach from shared memory."))  # noqa: F821



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from other import OtherObjectType
from sample import ObjectType


class MyObjectType(ObjectType):
    pass


class MyOtherObjectType(OtherObjectType):
    value = 10


obj = MyObjectType()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QAbstractListModel, QCoreApplication, QModelIndex


class MyModel (QAbstractListModel):
    def rowCount(self, parent=None):
        return 3


class TestBug706(unittest.TestCase):

    def mySlot(self, idx, start, end):
        self.start = start
        self.end = end

    def testIt(self):
        self.start = None
        self.end = None

        app = QCoreApplication([])  # noqa: F841
        model = MyModel()
        model.columnsAboutToBeInserted.connect(self.mySlot)
        model.columnsAboutToBeInserted.emit(QModelIndex(), 0, 1)
        self.assertEqual(self.start, 0)
        self.assertEqual(self.end, 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

from functools import partial
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal, QProcess

from helper.basicpyslotcase import BasicPySlotCase
from helper.usesqapplication import UsesQApplication


class MultipleSignalConnections(unittest.TestCase):
    '''Base class for multiple signal connection testing'''

    def run_many(self, signal, emitter, receivers, args=None):
        """Utility method to connect a list of receivers to a signal.
        sender - QObject that will emit the signal
        signal - string with the signal signature
        emitter - the callable that will trigger the signal
        receivers - list of BasicPySlotCase instances
        args - tuple with the arguments to be sent.
        """

        if args is None:
            args = tuple()
        for rec in receivers:
            rec.setUp()
            self.assertTrue(signal.connect(rec.cb))
            rec.args = tuple(args)

        emitter(*args)

        for rec in receivers:
            self.assertTrue(rec.called)


class PythonMultipleSlots(UsesQApplication, MultipleSignalConnections):
    '''Multiple connections to python signals'''

    def testPythonSignal(self):
        """Multiple connections to a python signal (short-circuit)"""

        class Sender(QObject):

            foobar = Signal(int)

        sender = Sender()
        receivers = [BasicPySlotCase() for x in range(10)]
        self.run_many(sender.foobar, partial(sender.foobar.emit),
                      receivers, (0, ))


class QProcessMultipleSlots(UsesQApplication, MultipleSignalConnections):
    '''Multiple connections to QProcess signals'''

    def testQProcessStarted(self):
        '''Multiple connections to QProcess.started()'''
        sender = QProcess()
        receivers = [BasicPySlotCase() for x in range(10)]

        def start_proc(*args):
            sender.start(sys.executable, ['-c', '""'])
            self.assertTrue(sender.waitForStarted())
            self.assertTrue(sender.waitForFinished())

        self.run_many(sender.started, start_proc, receivers)

    def testQProcessFinished(self):
        '''Multiple connections to QProcess.finished(int)'''
        sender = QProcess()
        receivers = [BasicPySlotCase() for x in range(10)]

        def start_proc(*args):
            sender.start(sys.executable, ['-c', '""'])
            self.assertTrue(sender.waitForStarted())
            self.assertTrue(sender.waitForFinished())

        self.run_many(sender.finished, start_proc, receivers, (0, QProcess.ExitStatus.NormalExit))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtWidgets import (QApplication, QStyledItemDelegate, QSpinBox,
                               QTableView)
from PySide6.QtGui import QStandardItemModel, Qt
from PySide6.QtCore import QModelIndex

"""PySide6 port of the widgets/itemviews/spinboxdelegate from Qt v6.x"""


#! [0]
class SpinBoxDelegate(QStyledItemDelegate):
    """A delegate that allows the user to change integer values from the model
       using a spin box widget. """

#! [0]
    def __init__(self, parent=None):
        super().__init__(parent)
#! [0]

#! [1]
    def createEditor(self, parent, option, index):
        editor = QSpinBox(parent)
        editor.setFrame(False)
        editor.setMinimum(0)
        editor.setMaximum(100)
        return editor
#! [1]

#! [2]
    def setEditorData(self, editor, index):
        value = index.model().data(index, Qt.ItemDataRole.EditRole)
        editor.setValue(value)
#! [2]

#! [3]
    def setModelData(self, editor, model, index):
        editor.interpretText()
        value = editor.value()
        model.setData(index, value, Qt.ItemDataRole.EditRole)
#! [3]

#! [4]
    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)
#! [4]


#! [main0]
if __name__ == '__main__':
    app = QApplication(sys.argv)

    model = QStandardItemModel(4, 2)
    tableView = QTableView()
    tableView.setModel(model)

    delegate = SpinBoxDelegate()
    tableView.setItemDelegate(delegate)
#! [main0]

    tableView.horizontalHeader().setStretchLastSection(True)

#! [main1]
    for row in range(4):
        for column in range(2):
            index = model.index(row, column, QModelIndex())
            value = (row + 1) * (column + 1)
            model.setData(index, value)
#! [main1] //# [main2]
#! [main2]

#! [main3]
    tableView.setWindowTitle("Spin Box Delegate")
    tableView.show()
    sys.exit(app.exec())
#! [main3]



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QTimeZone


class TestQTimeZone (unittest.TestCase):
    def testTimeZone(self):
        id = bytes('Europe/Berlin', "UTF-8")
        timeZone = QTimeZone(id)
        self.assertTrue(timeZone.isValid())
        self.assertEqual(timeZone.id(), id)
        name = timeZone.displayName(QTimeZone.TimeType.GenericTime,
                                    QTimeZone.NameType.DefaultName)
        self.assertTrue(name)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QByteArray'''

import ctypes
import os
import pickle
import struct
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)


from PySide6.QtCore import (QByteArray, QSettings, QObject, QDataStream,
                            QIODevice, qCompress, qUncompress)


class QByteArrayTestToNumber(unittest.TestCase):
    def testToNumberInt(self):
        obj = QByteArray(bytes('37', "UTF8"))
        self.assertEqual((37, True), obj.toInt())

    def testToNumberUShort(self):
        obj = QByteArray(bytes('37', "UTF8"))
        self.assertEqual((37, True), obj.toUShort())

    def testToNumberFloat(self):
        obj = QByteArray(bytes('37.109', "UTF8"))
        self.assertEqual((ctypes.c_float(37.109).value, True),
                         obj.toFloat())

    def testToNumberDouble(self):
        obj = QByteArray(bytes('37.109', "UTF8"))
        self.assertEqual((ctypes.c_double(37.109).value, True),
                         obj.toDouble())

    def testSetNum(self):
        b = QByteArray()
        b.setNum(int(-124124))
        self.assertEqual(b, "-124124")
        b = QByteArray()
        b.setNum(-124124)
        self.assertEqual(b, "-124124")
        b = QByteArray()
        b.setNum(-0.5)
        self.assertEqual(b, "-0.5")

    def testNumber(self):
        b = QByteArray.number(int(-124124))
        self.assertEqual(b, "-124124")
        b = QByteArray.number(-124124)
        self.assertEqual(b, "-124124")
        b = QByteArray.number(-0.5)
        self.assertEqual(b, "-0.5")

    def testAppend(self):
        b = QByteArray()
        b.append(bytes("A", "UTF8"))
        self.assertEqual(b.size(), 1)
        b.append(bytes("AB", "UTF8"))
        self.assertEqual(b.size(), 3)


class QByteArraySplit(unittest.TestCase):
    '''Test case for QByteArray.split'''

    def testPathSeparator(self):
        # QByteArray.split('/')
        obj = QByteArray(bytes(unittest.__file__, "UTF8"))
        self.assertEqual(obj.split('/'), unittest.__file__.split('/'))


class QByteArrayData(unittest.TestCase):

    '''Test case for QByteArray.data'''

    def testData(self):
        url = QByteArray(bytes("http://pyside.org", "UTF8"))
        self.assertEqual(url.data(), bytes("http://pyside.org", "UTF8"))

    def testDataWithZeros(self):
        s1 = bytes("123\000321", "UTF8")
        ba = QByteArray(s1)
        s2 = ba.data()
        self.assertEqual(s1, s2)
        self.assertEqual(s1, ba)


class QByteArrayOperatorAtSetter(unittest.TestCase):
    '''Test case for operator QByteArray[] - __setitem__'''

    def testSetterString(self):
        '''QByteArray[x] = pythonstring'''
        obj = QByteArray(bytes('123456', "UTF8"))
        obj[1] = bytes('0', "UTF8")
        self.assertEqual(obj, QByteArray(bytes('103456', "UTF8")))


class QByteArrayOnQDataStream(unittest.TestCase):
    '''
    Bug PYSIDE-232
    '''
    def testIt(self):
        a = QByteArray()
        b = QDataStream(a, QIODevice.OpenModeFlag.WriteOnly)
        b.writeUInt16(5000)
        # The __repr__ not suppose to crash anymore
        self.assertNotEqual(repr(b), None)


class TestBug664(unittest.TestCase):
    '''
    QByteArray.data() should return correct data
    '''
    def testIt(self):
        a = QByteArray(bytes('hi 猫', "UTF-8"))
        self.assertEqual(repr(a), "PySide6.QtCore.QByteArray(b'hi \\xe7\\x8c\\xab')")


class QByteArrayOnQVariant(unittest.TestCase):
    def testQByteArrayOnQVariant(self):
        a = QSettings().value("some_prop", QByteArray())
        self.assertEqual(type(a), QByteArray)


class TestBug567(unittest.TestCase):
    '''
    QByteArray should support slices
    '''
    def testIt(self):
        ba = QByteArray(bytes('1234567890', "UTF8"))
        self.assertEqual(ba[2:4], '34')
        self.assertEqual(ba[:4], '1234')
        self.assertEqual(ba[4:], '567890')
        self.assertEqual(len(ba[4:1]), 0)
        self.assertEqual(ba[::-1], '0987654321')
        self.assertEqual(ba[::2], '13579')
        self.assertEqual(ba[::-2], '08642')
        self.assertEqual(ba[2:8:3], '36')


class TestPickler(unittest.TestCase):
    def testIt(self):
        ba = QByteArray(bytes("321\x00123", "UTF8"))
        output = pickle.dumps(str(ba))
        ba2 = pickle.loads(output)
        self.assertEqual(str(ba), str(ba2))


class QByteArrayBug720(unittest.TestCase):
    def testIt(self):
        ba = QByteArray(bytes("32\"1\x00123", "UTF8"))
        self.assertEqual(str(ba), str(bytes("32\"1\x00123", "UTF-8")))
        self.assertEqual(repr(ba), "PySide6.QtCore.QByteArray(b'32\"1\\x00123')")


class QByteArrayImplicitConvert(unittest.TestCase):
    def testString(self):
        # No implicit conversions from QByteArray to python string
        ba = QByteArray(bytes("object name", "UTF8"))
        obj = QObject()
        self.assertRaises(TypeError, obj.setObjectName, ba)


class QByteArraySliceAssignment(unittest.TestCase):
    def testIndexAssignment(self):
        a = QByteArray(bytes('abc', "UTF8"))
        a[0] = bytes('x', "UTF8")
        self.assertEqual(a[0], bytes('x', "UTF8"))

        def test_1():
            a[0] = bytes('xy', "UTF8")
        self.assertRaises(ValueError, test_1)

    def testSliceAssignmentBytes(self):
        b = QByteArray(bytes('0123456789', "UTF8"))
        b[2:8] = bytes('abcdef', "UTF8")
        self.assertEqual(b[2:8], bytes('abcdef', "UTF8"))
        # Delete behavior
        b[2:8] = None
        self.assertEqual(b, bytes('0189', "UTF8"))

        # number of slots and number of values doesn't match
        def test_2():
            b[2:8:2] = bytes('', "UTF8")
        self.assertRaises(ValueError, test_2)
        b = QByteArray(bytes('0123456789', "UTF8"))
        # reverse slice
        b[5:2:-1] = bytes('ABC', "UTF8")
        self.assertEqual(b, bytes('012CBA6789', "UTF8"))
        # step is not 1
        b[2:9:3] = bytes('XYZ', "UTF8")
        self.assertEqual(b, bytes('01XCBY67Z9', "UTF8"))
        b = QByteArray(bytes('0123456789', "UTF8"))
        b[9:2:-3] = bytes('XYZ', "UTF8")
        self.assertEqual(b, bytes('012Z45Y78X', "UTF8"))

    def testSliceAssignmentQByteArray(self):
        b = QByteArray(bytes('0123456789', "UTF8"))
        b[2:8] = QByteArray(bytes('abcdef', "UTF8"))
        self.assertEqual(b[2:8], bytes('abcdef', "UTF8"))
        # shrink
        b[2:8] = QByteArray(bytes('aaa', "UTF8"))
        self.assertEqual(b, bytes('01aaa89', "UTF8"))
        # expand
        b[2:5] = QByteArray(bytes('uvwxyz', "UTF8"))
        self.assertEqual(b, bytes('01uvwxyz89', "UTF8"))
        # Delete behavior
        b[2:8] = QByteArray()
        self.assertEqual(b, bytes('0189', "UTF8"))

        b = QByteArray(bytes('0123456789', "UTF8"))
        # reverse assignment
        b[5:2:-1] = QByteArray(bytes('ABC', "UTF8"))
        self.assertEqual(b, bytes('012CBA6789', "UTF8"))
        # step is not 1
        b[2:9:3] = QByteArray(bytes('XYZ', "UTF8"))
        self.assertEqual(b, bytes('01XCBY67Z9', "UTF8"))
        b = QByteArray(bytes('0123456789', "UTF8"))
        b[9:2:-3] = QByteArray(bytes('XYZ', "UTF8"))
        self.assertEqual(b, bytes('012Z45Y78X', "UTF8"))

    def testSliceAssignmentByteArray(self):
        b = QByteArray(bytes('0123456789', "UTF8"))
        # replace
        b[2:8] = bytearray(bytes('abcdef', "UTF8"))
        self.assertEqual(b[2:8], bytes('abcdef', "UTF8"))
        # shrink
        b[2:8] = bytearray(bytes('aaa', "UTF8"))
        self.assertEqual(b, bytes('01aaa89', "UTF8"))
        # expand
        b[2:5] = bytearray(bytes('uvwxyz', "UTF8"))
        self.assertEqual(b, bytes('01uvwxyz89', "UTF8"))
        # Delete behavior
        b[2:8] = bytearray(bytes('', "UTF8"))
        self.assertEqual(b, bytes('0189', "UTF8"))

        b = QByteArray(bytes('0123456789', "UTF8"))
        # reverse assignment
        b[5:2:-1] = bytearray(bytes('ABC', "UTF8"))
        self.assertEqual(b, bytes('012CBA6789', "UTF8"))
        # step is not 1
        b[2:9:3] = bytearray(bytes('XYZ', "UTF8"))
        self.assertEqual(b, bytes('01XCBY67Z9', "UTF8"))
        b = QByteArray(bytes('0123456789', "UTF8"))
        b[9:2:-3] = bytearray(bytes('XYZ', "UTF8"))
        self.assertEqual(b, bytes('012Z45Y78X', "UTF8"))

    def testBufferProtocol(self):
        orig_bytes = bytes('0123456789', "UTF8")
        byte_array = QByteArray(orig_bytes)
        actual_bytes = bytes(byte_array)
        self.assertEqual(orig_bytes, actual_bytes)

    def testUnpack(self):
        b = QByteArray(b'\x19\x00\x00\x00\xc4\t\x00\x00')
        t = struct.unpack('<ii', b)
        self.assertEqual(len(t), 2)
        self.assertEqual(t[0], 25)
        self.assertEqual(t[1], 2500)


class QCompressTest(unittest.TestCase):
    def testQByteArrayCompression(self):
        """Compress/uncompress a QByteArray."""
        data = bytes(10 * 'long redundant sentence bla bla', "UTF8")
        ba = QByteArray(data)
        compressed = qCompress(ba)
        self.assertTrue(len(compressed) < len(data))
        uncompressed = qUncompress(compressed)
        self.assertEqual(uncompressed, data)

    def testBufferCompression(self):
        """Compress/uncompress portions of bytes without converting to
           QByteArray."""
        data = bytes(10 * 'long redundant sentence bla bla', "UTF8")
        used_len = int(len(data) / 2)
        compressed = qCompress(data, used_len, -1)
        self.assertTrue(len(compressed) < used_len)
        uncompressed = qUncompress(compressed.data(), len(compressed))
        self.assertEqual(uncompressed, data[:used_len])


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QModelIndex, QStringListModel


class TestQAbstractItemModelSignals(unittest.TestCase):
    def sigCallback(self, index, r, c):
        self._called = True

    def testSignals(self):
        self._called = False
        m = QStringListModel()
        m.rowsAboutToBeInserted[QModelIndex, int, int].connect(self.sigCallback)
        m.insertRows(0, 3)
        self.assertTrue(self._called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2018 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

from pathlib import Path

from ..log import log
from ..config import config
from ..options import OPTION
from ..utils import (copy_icu_libs, copydir, copyfile, find_files_using_glob,
                     linux_patch_executable)
from .. import PYSIDE, PYSIDE_UNIX_BUNDLED_TOOLS


def prepare_standalone_package_linux(pyside_build, _vars, cross_build=False, is_android=False):
    built_modules = _vars['built_modules']

    constrain_modules = None
    copy_plugins = True
    copy_qml = True
    copy_translations = True
    copy_qt_conf = True

    log.info("Copying files...")

    if config.is_internal_shiboken_generator_build():
        constrain_modules = ["Core", "Network", "Xml", "XmlPatterns"]
        copy_plugins = False
        copy_qml = False
        copy_translations = False
        copy_qt_conf = False

    # <qt>/lib/* -> <setup>/{st_package_name}/Qt/lib
    destination_dir = Path("{st_build_dir}/{st_package_name}".format(**_vars))
    destination_qt_dir = destination_dir / "Qt"
    destination_qt_lib_dir = destination_qt_dir / "lib"

    # android libs does not have the Qt major version
    if is_android:
        lib_regex = 'libQt6*.so*'
    else:
        lib_regex = 'libQt6*.so.?'

    accepted_modules = [lib_regex]
    if constrain_modules:
        accepted_modules = [f"libQt6{module}*.so.?" if not is_android else f"libQt6{module}*.so*"
                            for module in constrain_modules]
    accepted_modules.append("libicu*.so.??")

    if is_android:
        accepted_modules.append("*-android-dependencies.xml")

    copydir("{qt_lib_dir}", destination_qt_lib_dir,
            _filter=accepted_modules,
            recursive=False, _vars=_vars, force_copy_symlinks=True)

    if not cross_build and not is_android:
        # Check if ICU libraries were copied over to the destination
        # Qt libdir.
        maybe_icu_libs = find_files_using_glob(destination_qt_lib_dir, "libicu*")

        # If no ICU libraries are present in the Qt libdir (like when
        # Qt is built against system ICU, or in the Coin CI where ICU
        # libs are in a different directory) try to find out / resolve
        # which ICU libs are used by QtCore (if used at all) using a
        # custom written ldd (non-cross build only), and copy the ICU
        # libs to the Pyside Qt dir if necessary.
        # We choose the QtCore lib to inspect, by
        # checking which QtCore library the shiboken6 executable uses.
        if not maybe_icu_libs:
            copy_icu_libs(pyside_build._patchelf_path, destination_qt_lib_dir)

    # Set RPATH for Qt libs.
    if not is_android:
        pyside_build.update_rpath_for_linux_qt_libraries(destination_qt_lib_dir)

    # Patching designer to use the Qt libraries provided in the wheel
    if config.is_internal_pyside_build() and not OPTION['NO_QT_TOOLS']:

        for tool in PYSIDE_UNIX_BUNDLED_TOOLS:
            tool_path = destination_dir / tool
            linux_patch_executable(pyside_build._patchelf_path, tool_path)

    if pyside_build.is_webengine_built(built_modules):
        copydir("{qt_data_dir}/resources",
                destination_qt_dir / "resources",
                _filter=None,
                recursive=False,
                _vars=_vars)

    if copy_plugins:
        is_pypy = "pypy" in pyside_build.build_classifiers
        # <qt>/plugins/* -> <setup>/{st_package_name}/Qt/plugins
        plugins_target = destination_qt_dir / "plugins"
        copydir("{qt_plugins_dir}", plugins_target,
                _filter=["*.so"],
                recursive=True,
                _vars=_vars)
        if not is_pypy and not is_android:
            copydir("{install_dir}/plugins/designer",
                    plugins_target / "designer",
                    _filter=["*.so"],
                    recursive=False,
                    _vars=_vars)

        copied_plugins = pyside_build.get_shared_libraries_in_path_recursively(
            plugins_target)
        if not is_android:
            pyside_build.update_rpath_for_linux_plugins(copied_plugins)

    if copy_qml:
        # <qt>/qml/* -> <setup>/{st_package_name}/Qt/qml
        qml_plugins_target = destination_qt_dir / "qml"
        copydir("{qt_qml_dir}",
                qml_plugins_target,
                _filter=None,
                force=False,
                recursive=True,
                ignore=["*.debug"],
                _vars=_vars)
        copied_plugins = pyside_build.get_shared_libraries_in_path_recursively(
            qml_plugins_target)
        if not is_android:
            pyside_build.update_rpath_for_linux_plugins(
                copied_plugins,
                qt_lib_dir=destination_qt_lib_dir,
                is_qml_plugin=True)

    if copy_translations:
        # <qt>/translations/* ->
        # <setup>/{st_package_name}/Qt/translations
        copydir("{qt_translations_dir}",
                destination_qt_dir / "translations",
                _filter=["*.qm", "*.pak"],
                force=False,
                _vars=_vars)

    if copy_qt_conf:
        # Copy the qt.conf file to libexec.
        qt_libexec_path = destination_qt_dir / "libexec"
        if not qt_libexec_path.is_dir():
            qt_libexec_path.mkdir(parents=True)
        copyfile(f"{{build_dir}}/{PYSIDE}/{{st_package_name}}/qt.conf",
                 qt_libexec_path, _vars=_vars)



# Copyright (C) 2011 Arun Srinivasan <rulfzid@gmail.com>
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot
from PySide6.QtGui import QAction
from PySide6.QtWidgets import (QMainWindow, QFileDialog, QApplication)

from addresswidget import AddressWidget


class MainWindow(QMainWindow):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._address_widget = AddressWidget()
        self.setCentralWidget(self._address_widget)
        self.create_menus()
        self.setWindowTitle("Address Book")

    def create_menus(self):
        # Create the main menuBar menu items
        file_menu = self.menuBar().addMenu("&File")
        tool_menu = self.menuBar().addMenu("&Tools")

        # Populate the File menu
        self.open_action = self.create_action("&Open...", file_menu, self.open_file)
        self.save_action = self.create_action("&Save As...", file_menu, self.save_file)
        file_menu.addSeparator()
        self.exit_action = self.create_action("E&xit", file_menu, self.close)

        # Populate the Tools menu
        self.add_action = self.create_action(
            "&Add Entry...", tool_menu, self._address_widget.add_entry)
        self._edit_action = self.create_action(
            "&Edit Entry...", tool_menu, self._address_widget.edit_entry)
        tool_menu.addSeparator()
        self._remove_action = self.create_action(
            "&Remove Entry", tool_menu, self._address_widget.remove_entry)

        # Disable the edit and remove menu items initially, as there are
        # no items yet.
        self._edit_action.setEnabled(False)
        self._remove_action.setEnabled(False)

        # Wire up the updateActions slot
        self._address_widget.selection_changed.connect(self.update_actions)

    def create_action(self, text, menu, slot):
        """ Helper function to save typing when populating menus
            with action.
        """
        action = QAction(text, self)
        menu.addAction(action)
        action.triggered.connect(slot)
        return action

    # Quick  gotcha:
    #
    # QFiledialog.getOpenFilename and QFileDialog.get.SaveFileName don't
    # behave in PySide6 as they do in Qt, where they return a QString
    # containing the filename.
    #
    # In PySide6, these functions return a tuple: (filename, filter)

    @Slot()
    def open_file(self):
        filename, _ = QFileDialog.getOpenFileName(self)
        if filename:
            self._address_widget.read_from_file(filename)

    @Slot()
    def save_file(self):
        filename, _ = QFileDialog.getSaveFileName(self)
        if filename:
            self._address_widget.write_to_file(filename)

    def update_actions(self, selection):
        """ Only allow the user to remove or edit an item if an item
            is actually selected.
        """
        indexes = selection.indexes()

        if len(indexes) > 0:
            self._remove_action.setEnabled(True)
            self._edit_action.setEnabled(True)
        else:
            self._remove_action.setEnabled(False)
            self._edit_action.setEnabled(False)


if __name__ == "__main__":
    """ Run the application. """
    import sys
    app = QApplication(sys.argv)
    mw = MainWindow()
    mw.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QCoreApplication, QEvent, QThread


class MyEvent(QEvent):
    def __init__(self, i):
        print("TYPE:", type(QEvent.Type.User))
        super().__init__(QEvent.Type(QEvent.Type.User))
        self.i = i


class MyThread (QThread):
    def __init__(self, owner):
        super().__init__()
        self.owner = owner

    def run(self):
        for i in range(3):
            e = MyEvent(i)
            QCoreApplication.postEvent(self.owner, e)


class MyBaseObject(QObject):
    def __init__(self):
        super().__init__()
        self.events = []
        self.t = MyThread(self)
        self.t.start()

    def customEvent(self, event):
        self.events.append(event)
        if len(self.events) == 3:
            self.t.wait()
            self.app.quit()


class CheckForEventsTypes(unittest.TestCase):
    def testTypes(self):
        o = MyBaseObject()
        o.app = QCoreApplication(sys.argv)
        o.app.exec()
        for e in o.events:
            self.assertTrue(isinstance(e, MyEvent))
        o.app = None


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

modules_to_test = {
    # 6.0
    'QtCore': 'qtcore-module.html',
    'QtGui': 'qtgui-module.html',
    'QtNetwork': 'qtnetwork-module.html',
    'QtQml': 'qtqml-module.html',
    'QtQuick': 'qtquick-module.html',
    'QtQuickWidgets': 'qtquickwidgets-module.html',
    # Broken in 6.5.0
    #'QtQuickControls2': 'qtquickcontrols-module.html',
    'QtQuickControls2': 'qtquickcontrols2-module.html',
    'QtSql': 'qtsql-module.html',
    'QtWidgets': 'qtwidgets-module.html',
    'QtConcurrent': 'qtconcurrent-module.html',
    'QtDBus': 'qtdbus-module.html',
    'QtHelp': 'qthelp-module.html',
    'QtOpenGL': 'qtopengl-module.html',
    'QtPrintSupport': 'qtprintsupport-module.html',
    'QtSvg': 'qtsvg-module.html',
    'QtSvgWidgets': 'qtsvgwidgets-module.html',
    'QtUiTools': 'qtuitools-module.html',
    'QtXml': 'qtxml-module.html',
    'QtQuickTest': 'qtquicktest-module.html',
    'QtTest': 'qttest-module.html',
    # Deprecated in 6.8
    #'Qt3DCore': 'qt3dcore-module.html',
    #'Qt3DInput': 'qt3dinput-module.html',
    #'Qt3DLogic': 'qt3dlogic-module.html',
    #'Qt3DRender': 'qt3drender-module.html',
    #'Qt3DAnimation': 'qt3danimation-module.html',
    #'Qt3DExtras': 'qt3dextras-module.html',
    'QtNetworkAuth': 'qtnetworkauth-module.html',
    'QtStateMachine': 'qtstatemachine-module.html',
    'QtCoap': 'qtcoap-module.html',
    'QtMqtt': 'qtmqtt-module.html',
    'QtOpcUa': 'qtopcua-module.html',
    'QtVirtualKeyboard': 'qtvirtualkeyboard-module.html',
    'QtWaylandCompositor': 'qtwaylandcompositor-module.html',
    'QtWebView': 'qtwebview-module.html',

    # 6.1
    'QtScxml': 'qtscxml-module.html',

    # 6.2
    'QtBluetooth': 'qtbluetooth-module.html',
    'QtPositioning': 'qtpositioning-module.html',
    'QtMultimedia': 'qtmultimedia-module.html',
    'QtRemoteObjects': 'qtremoteobjects-module.html',
    'QtSensors': 'qtsensors-module.html',
    'QtSerialPort': 'qtserialport-module.html',
    'QtWebChannel': 'qtwebchannel-module.html',
    'QtWebEngineCore': 'qtwebenginecore-module.html',
    'QtWebEngineQuick': 'qtwebenginequick-module.html',
    'QtWebEngineWidgets': 'qtwebenginewidgets-module.html',
    'QtWebSockets': 'qtwebsockets-module.html',
    'QtHttpServer': 'qthttpserver-module.html',

    #  6.3
    #'QtSpeech':  'qtspeech-module.html',
    'QtMultimediaWidgets': 'qtmultimediawidgets-module.html',
    'QtNfc': 'qtnfc-module.html',
    'QtQuick3D': 'qtquick3d-module.html',

    # 6.4
    'QtPdf': 'qtpdf-module.html',  # this include qtpdfwidgets
    'QtSpatialAudio': 'qtspatialaudio-module.html',

    # 6.5
    'QtSerialBus': 'qtserialbus-module.html',
    'QtTextToSpeech': 'qttexttospeech-module.html',
    'QtLocation': 'qtlocation-module.html',
    'QtGRPC': 'qtgrpc-module.html',
    'QtPhotobuf': 'qtprotobuf-module.html',

    # 6.6
    'QtGraphs': 'qtgraphs-module.html',

    # 6.7
    'QtCharts': 'qtcharts-module-qtcharts.html',
    'QtDataVisualization': 'qtdatavisualization-module-qtdatavis.html',
}

types_to_ignore = {
    # QtCore
    'QFlag',
    'QFlags',
    'QGlobalStatic',
    'QDebug',
    'QDebugStateSaver',
    'QMetaObject.Connection',
    'QPointer',
    'QAssociativeIterable',
    'QSequentialIterable',
    'QStaticPlugin',
    'QChar',
    'QLatin1Char',
    'QHash',
    'QMultiHash',
    'QLinkedList',
    'QList',
    'QMap',
    'QMultiMap',
    'QMap.key_iterator',
    'QPair',
    'QQueue',
    'QScopedArrayPointer',
    'QScopedPointer',
    'QScopedValueRollback',
    'QMutableSetIterator',
    'QSet',
    'QSet.const_iterator',
    'QSet.iterator',
    'QExplicitlySharedDataPointer',
    'QSharedData',
    'QSharedDataPointer',
    'QEnableSharedFromThis',
    'QSharedPointer',
    'QWeakPointer',
    'QStack',
    'QLatin1String',
    'QString',
    'QStringRef',
    'QStringList',
    'QStringMatcher',
    'QVarLengthArray',
    'QVector',
    'QFutureIterator',
    'QHashIterator',
    'QMutableHashIterator',
    'QLinkedListIterator',
    'QMutableLinkedListIterator',
    'QListIterator',
    'QMutableListIterator',
    'QMapIterator',
    'QMutableMapIterator',
    'QSetIterator',
    'QMutableVectorIterator',
    'QVectorIterator',
    # QtGui
    'QIconEnginePlugin',
    'QImageIOPlugin',
    'QGenericPlugin',
    'QGenericPluginFactory',
    'QGenericMatrix',
    'QOpenGLExtraFunctions',
    # QtWidgets
    'QItemEditorCreator',
    'QStandardItemEditorCreator',
    'QStylePlugin',
    # QtSql
    'QSqlDriverCreator',
    'QSqlDriverPlugin',
}

# Split modules on PySide
split_modules = {
    'QtSvg': 'QtSvgWidgets',
    'QtPdf': 'QtPdfWidgets',
    'QtOpenGL': 'QtOpenGLWidgets',
    'QtGraphs': 'QtGraphsWidgets',
}



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QBuffer


class TestBug938 (unittest.TestCase):

    def testIt(self):
        b = QBuffer()
        b.open(QBuffer.OpenModeFlag.WriteOnly)
        b.write(bytes("\x0023\x005", "UTF-8"))
        b.close()
        self.assertEqual(b.buffer().size(), 5)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import (QDialog, QFileDialog,
                               QPlainTextEdit)
from PySide6.QtGui import QAction, QGuiApplication, QIcon, QKeySequence
from PySide6.QtCore import QDir, QFile, QTextStream, Qt, Slot

from abstractviewer import AbstractViewer


class TxtViewer(AbstractViewer):
    def __init__(self):
        super().__init__()
        self.uiInitialized.connect(self.setupTxtUi)

    def init(self, file, parent, mainWindow):
        self._textEdit = QPlainTextEdit(parent)
        super().init(file, self._textEdit, mainWindow)

    def viewerName(self):
        return "TxtViewer"

    def supportedMimeTypes(self):
        return ["text/plain"]

    @Slot()
    def setupTxtUi(self):
        editMenu = self.addMenu("Edit")
        editToolBar = self.addToolBar("Edit")
        cutIcon = QIcon.fromTheme(QIcon.ThemeIcon.EditCut,
                                  QIcon(":/demos/documentviewer/images/cut.png"))
        cutAct = QAction(cutIcon, "Cut", self)
        cutAct.setShortcuts(QKeySequence.StandardKey.Cut)
        cutAct.setStatusTip("Cut the current selection's contents to the clipboard")
        cutAct.triggered.connect(self._textEdit.cut)
        editMenu.addAction(cutAct)
        editToolBar.addAction(cutAct)

        copyIcon = QIcon.fromTheme(QIcon.ThemeIcon.EditCopy,
                                   QIcon(":/demos/documentviewer/images/copy.png"))
        copyAct = QAction(copyIcon, "Copy", self)
        copyAct.setShortcuts(QKeySequence.StandardKey.Copy)
        copyAct.setStatusTip("Copy the current selection's contents to the clipboard")
        copyAct.triggered.connect(self._textEdit.copy)
        editMenu.addAction(copyAct)
        editToolBar.addAction(copyAct)

        pasteIcon = QIcon.fromTheme(QIcon.ThemeIcon.EditPaste,
                                    QIcon(":/demos/documentviewer/images/paste.png"))
        pasteAct = QAction(pasteIcon, "Paste", self)
        pasteAct.setShortcuts(QKeySequence.StandardKey.Paste)
        pasteAct.setStatusTip("Paste the clipboard's contents into the current selection")
        pasteAct.triggered.connect(self._textEdit.paste)
        editMenu.addAction(pasteAct)
        editToolBar.addAction(pasteAct)

        self.menuBar().addSeparator()

        cutAct.setEnabled(False)
        copyAct.setEnabled(False)
        self._textEdit.copyAvailable.connect(cutAct.setEnabled)
        self._textEdit.copyAvailable.connect(copyAct.setEnabled)

        self.openFile()

        self._textEdit.textChanged.connect(self._textChanged)
        self._uiAssets_back.triggered.connect(self._back)
        self._uiAssets_forward.triggered.connect(self._forward)

    @Slot()
    def _textChanged(self):
        self.maybeSetPrintingEnabled(self.hasContent())

    @Slot()
    def _back(self):
        bar = self._textEdit.verticalScrollBar()
        if bar.value() > bar.minimum():
            bar.setValue(bar.value() - 1)

    @Slot()
    def _forward(self):
        bar = self._textEdit.verticalScrollBar()
        if bar.value() < bar.maximum():
            bar.setValue(bar.value() + 1)

    def openFile(self):
        type = "open"
        file_name = QDir.toNativeSeparators(self._file.fileName())
        if not self._file.open(QFile.OpenModeFlag.ReadOnly
                               | QFile.OpenModeFlag.Text):
            err = self._file.errorString()
            self.statusMessage(f"Cannot read file {file_name}:\n{err}.", type)
            return

        in_str = QTextStream(self._file)
        QGuiApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        if self._textEdit.toPlainText():
            self._textEdit.clear()
            self.disablePrinting()

        self._textEdit.setPlainText(in_str.readAll())
        QGuiApplication.restoreOverrideCursor()

        self.statusMessage(f"File {file_name} loaded.", type)
        self.maybeEnablePrinting()

    def hasContent(self):
        return bool(self._textEdit.toPlainText())

    def printDocument(self, printer):
        if not self.hasContent():
            return

        self._textEdit.print_(printer)

    def saveFile(self, file):
        file_name = QDir.toNativeSeparators(self._file.fileName())
        errorMessage = ""
        QGuiApplication.setOverrideCursor(Qt.CursorShape.WaitCursor)
        if file.open(QFile.OpenModeFlag.WriteOnly | QFile.OpenModeFlag.Text):
            out = QTextStream(file)
            out << self._textEdit.toPlainText()
        else:
            error = file.errorString()
            errorMessage = f"Cannot open file {file_name} for writing:\n{error}."
        QGuiApplication.restoreOverrideCursor()

        if errorMessage:
            self.statusMessage(errorMessage)
            return False

        self.statusMessage(f"File {file_name} saved")
        return True

    def saveDocumentAs(self):
        dialog = QFileDialog(self.mainWindow())
        dialog.setWindowModality(Qt.WindowModal)
        dialog.setAcceptMode(QFileDialog.AcceptMode.AcceptSave)
        if dialog.exec() != QDialog.DialogCode.Accepted:
            return False

        files = dialog.selectedFiles()
        self._file.setFileName(files[0])
        return self.saveDocument()



painter = QPainter(self)
font = painter.font()
font.setPixelSize(48)
painter.setFont(font)
rectangle = QRect(0, 0, 100, 50)
boundingRect = painter.drawText(rectangle, 0, "Hello")
pen = painter.pen()
pen.setStyle(Qt.DotLine)
painter.setPen(pen)
painter.drawRect(boundingRect.adjusted(0, 0, -pen.width(), -pen.width()))
pen.setStyle(Qt.DashLine)
painter.setPen(pen)
painter.drawRect(rectangle.adjusted(0, 0, -pen.width(), -pen.width()))



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests calling Str constructor using a Number parameter, being that number defines
   a cast operator to Str.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Str
from other import Number


class NewCtorOperatorTest(unittest.TestCase):
    '''Tests calling Str constructor using a Number parameter, being that number
       defines a cast operator to Str.'''

    def testNumber(self):
        '''Basic test to see if the Number class is Ok.'''
        value = 123
        num = Number(value)
        self.assertEqual(num.value(), value)

    def testStrCtorWithNumberArgument(self):
        '''Try to build a Str from 'sample' module with a Number argument from 'other' module.'''
        value = 123
        num = Number(value)
        string = Str(num)  # noqa: F841


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from sample import ObjectTypeOperators


class ObjectTypeOperatorsTest(unittest.TestCase):

    def testIt(self):
        a = ObjectTypeOperators("a")
        b = ObjectTypeOperators("b")
        self.assertFalse(a == b)
        self.assertEqual(a, a < b)

        # this should change a.key() and return nothing.
        self.assertEqual(None, a > b)
        self.assertEqual(a.key(), "aoperator>")

    def testPointerOpeators(self):
        a = ObjectTypeOperators("a")
        b = ObjectTypeOperators("b")  # noqa: F841
        self.assertEqual(a + "bc", "abc")
        self.assertEqual("bc" + a, "bca")
        self.assertEqual("a", a)
        self.assertEqual(a, "a")

    def testOperatorInjection(self):
        a = ObjectTypeOperators("a")
        self.assertNotEqual(a, "b")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from sample import Reference


class TestLackOfDereferenceOperators (unittest.TestCase):
    def testIf(self):
        r = Reference()
        self.assertFalse(hasattr(r, "__mul__"))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtMultimedia import QMediaPlayer, QtAudio
from PySide6.QtWidgets import (QComboBox, QHBoxLayout, QSizePolicy, QSlider, QStyle,
                               QToolButton, QWidget)
from PySide6.QtGui import QPalette
from PySide6.QtCore import qFuzzyCompare, Qt, Signal, Slot


class PlayerControls(QWidget):

    play = Signal()
    pause = Signal()
    stop = Signal()
    previous = Signal()
    changeVolume = Signal(float)
    changeMuting = Signal(bool)
    changeRate = Signal(float)

    def __init__(self, parent=None):
        super().__init__(parent)

        style = self.style()
        self.m_playerState = QMediaPlayer.PlaybackState.StoppedState
        self.m_playerMuted = False

        self.m_playButton = QToolButton(self)
        self.m_playButton.setIcon(style.standardIcon(QStyle.StandardPixmap.SP_MediaPlay))
        self.m_playButton.setToolTip("Play")
        self.m_playButton.clicked.connect(self.playClicked)

        self.m_pauseButton = QToolButton(self)
        self.m_pauseButton.setIcon(style.standardIcon(QStyle.StandardPixmap.SP_MediaPause))
        self.m_pauseButton.setToolTip("Pause")
        self.m_pauseButton.clicked.connect(self.pauseClicked)

        self.m_stopButton = QToolButton(self)
        self.m_stopButton.setIcon(style.standardIcon(QStyle.StandardPixmap.SP_MediaStop))
        self.m_stopButton.setToolTip("Stop")
        self.m_stopButton.clicked.connect(self.stop)

        self.m_previousButton = QToolButton(self)
        self.m_previousButton.setIcon(style.standardIcon(QStyle.StandardPixmap.SP_MediaSkipBackward))  # noqa: E501
        self.m_previousButton.setToolTip("Rewind")
        self.m_previousButton.clicked.connect(self.previous)

        self.m_muteButton = QToolButton(self)
        self.m_muteButton.setIcon(style.standardIcon(QStyle.StandardPixmap.SP_MediaVolume))
        self.m_muteButton.setToolTip("Mute")
        self.m_muteButton.clicked.connect(self.muteClicked)

        self.m_volumeSlider = QSlider(Qt.Orientation.Horizontal, self)
        self.m_volumeSlider.setRange(0, 100)
        sp = self.m_volumeSlider.sizePolicy()
        sp.setHorizontalPolicy(QSizePolicy.Policy.MinimumExpanding)
        self.m_volumeSlider.setSizePolicy(sp)
        self.m_volumeSlider.valueChanged.connect(self.onVolumeSliderValueChanged)

        self.m_rateBox = QComboBox(self)
        self.m_rateBox.setToolTip("Rate")
        self.m_rateBox.addItem("0.5x", 0.5)
        self.m_rateBox.addItem("1.0x", 1.0)
        self.m_rateBox.addItem("2.0x", 2.0)
        self.m_rateBox.setCurrentIndex(1)

        self.m_rateBox.activated.connect(self.updateRate)

        self._doSetState(QMediaPlayer.PlaybackState.StoppedState, True)

        layout = QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.m_stopButton)
        layout.addWidget(self.m_previousButton)
        layout.addWidget(self.m_pauseButton)
        layout.addWidget(self.m_playButton)
        layout.addWidget(self.m_muteButton)
        layout.addWidget(self.m_volumeSlider)
        layout.addWidget(self.m_rateBox)

    def state(self):
        return self.m_playerState

    @Slot(QMediaPlayer.PlaybackState)
    def setState(self, state):
        self._doSetState(state, False)

    def _doSetState(self, state, force):
        if state != self.m_playerState or force:
            self.m_playerState = state

            baseColor = self.palette().color(QPalette.ColorRole.Base)
            inactiveStyleSheet = f"background-color: {baseColor.name()}"
            defaultStyleSheet = ""

            if state == QMediaPlayer.PlaybackState.StoppedState:
                self.m_stopButton.setStyleSheet(inactiveStyleSheet)
                self.m_playButton.setStyleSheet(defaultStyleSheet)
                self.m_pauseButton.setStyleSheet(defaultStyleSheet)
            elif state == QMediaPlayer.PlaybackState.PlayingState:
                self.m_stopButton.setStyleSheet(defaultStyleSheet)
                self.m_playButton.setStyleSheet(inactiveStyleSheet)
                self.m_pauseButton.setStyleSheet(defaultStyleSheet)
            elif state == QMediaPlayer.PlaybackState.PausedState:
                self.m_stopButton.setStyleSheet(defaultStyleSheet)
                self.m_playButton.setStyleSheet(defaultStyleSheet)
                self.m_pauseButton.setStyleSheet(inactiveStyleSheet)

    def volume(self):
        linearVolume = QtAudio.convertVolume(self.m_volumeSlider.value() / 100.0,
                                             QtAudio.VolumeScale.LogarithmicVolumeScale,
                                             QtAudio.VolumeScale.LinearVolumeScale)
        return linearVolume

    @Slot("float")
    def setVolume(self, volume):
        logarithmicVolume = QtAudio.convertVolume(volume, QtAudio.VolumeScale.LinearVolumeScale,
                                                  QtAudio.VolumeScale.LogarithmicVolumeScale)
        self.m_volumeSlider.setValue(round(logarithmicVolume * 100.0))

    def isMuted(self):
        return self.m_playerMuted

    @Slot(bool)
    def setMuted(self, muted):
        if muted != self.m_playerMuted:
            self.m_playerMuted = muted
            sp = (QStyle.StandardPixmap.SP_MediaVolumeMuted
                  if muted else QStyle.StandardPixmap.SP_MediaVolume)
            self.m_muteButton.setIcon(self.style().standardIcon(sp))

    @Slot()
    def playClicked(self):
        self.play.emit()

    @Slot()
    def pauseClicked(self):
        self.pause.emit()

    @Slot()
    def muteClicked(self):
        self.changeMuting.emit(not self.m_playerMuted)

    def playbackRate(self):
        return self.m_rateBox.itemData(self.m_rateBox.currentIndex())

    def setPlaybackRate(self, rate):
        for i in range(0, self.m_rateBox.count()):
            if qFuzzyCompare(rate, self.m_rateBox.itemData(i)):
                self.m_rateBox.setCurrentIndex(i)
                return

        self.m_rateBox.addItem(f"{rate}x", rate)
        self.m_rateBox.setCurrentIndex(self.m_rateBox.count() - 1)

    @Slot()
    def updateRate(self):
        self.changeRate.emit(self.playbackRate())

    @Slot()
    def onVolumeSliderValueChanged(self):
        self.changeVolume.emit(self.volume())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

'''
Unit tests for attribute cache in Python 3

This is the original code from the bug report PYSIDE-60
'''

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
sys.path.append(os.fspath(Path(__file__).resolve().parents[1] / "util"))
from init_paths import init_test_paths  # noqa: E402
init_test_paths()

from PySide6.QtCore import QObject  # noqa: E402


class A(QObject):
    instance = 1

    @classmethod
    def test(cls):
        cls.instance
        cls.instance = cls()
        assert "<__main__.A(0x" in repr(cls.__dict__['instance'])
        assert "<__main__.A(0x" in repr(cls.instance)
        assert "<__main__.A(0x" in repr(type.__getattribute__(cls, 'instance'))


if __name__ == "__main__":
    A.test()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QResource usage'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QByteArray, QFile, QIODevice
import resources_mc  # noqa: F401


class ResourcesUsage(unittest.TestCase):
    '''Test case for resources usage'''

    def testPhrase(self):
        # Test loading of quote.txt resource
        file = Path(__file__).resolve().parent / 'quoteEnUS.txt'
        self.assertTrue(file.is_file())
        orig = QByteArray(file.read_bytes())
        # In case the file is checked out in 'crlf' mode, strip '\r'
        # since we read binary.
        if sys.platform == 'win32':
            carriage_return = orig.indexOf('\r')
            if carriage_return != -1:
                orig.remove(carriage_return, 1)

        f = QFile(':/quote.txt')  # |QIODevice.Text
        self.assertTrue(f.open(QIODevice.OpenModeFlag.ReadOnly), f.errorString())
        copy = f.readAll()
        f.close()
        self.assertEqual(orig, copy)

    def testImage(self):
        # Test loading of sample.png resource
        file = Path(__file__).resolve().parent / 'sample.png'
        self.assertTrue(file.is_file())
        orig = file.read_bytes()

        f = QFile(':/sample.png')
        self.assertTrue(f.open(QIODevice.OpenModeFlag.ReadOnly), f.errorString())
        copy = f.readAll()
        f.close()
        self.assertEqual(len(orig), len(copy))
        self.assertEqual(orig, copy)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtWidgets import QApplication

from mainwindow import MainWindow

"""PySide6 port of the serialport/terminal example from Qt v6.x"""


if __name__ == "__main__":
    a = QApplication(sys.argv)
    w = MainWindow()
    w.show()
    sys.exit(a.exec())



# Copyright (C) 2018 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

from setuptools import Command

import sys
import logging
from pathlib import Path

from .log import log, LogLevel
from .qtinfo import QtInfo
from .utils import memoize, which, Singleton

_AVAILABLE_MKSPECS = ["ninja", "msvc", "mingw"] if sys.platform == "win32" else ["ninja", "make"]


# Global options not which are not part of the commands
ADDITIONAL_OPTIONS = """
Additional options:
  --limited-api                        Use Limited API [yes/no]
  --macos-use-libc++                   Use libc++ on macOS
  --snapshot-build                     Snapshot build
  --package-timestamp                  Package Timestamp
  --cmake-toolchain-file               Path to CMake toolchain to enable cross-compiling
  --shiboken-host-path                 Path to host shiboken package when cross-compiling
  --qt-host-path                       Path to host Qt installation when cross-compiling
  --disable-pyi                        Disable .pyi file generation
  --skip-mypy-test                     Skip the mypy test which can take long in debug builds
"""


def _warn_multiple_option(option):
    log.warning(f'Option "{option}" occurs multiple times on the command line.')


def _warn_deprecated_option(option, replacement=None):
    w = f'Option "{option}" is deprecated and may be removed in a future release.'
    if replacement:
        w = f'{w}\nUse "{replacement}" instead.'
    log.warning(w)


class Options(metaclass=Singleton):
    def __init__(self):

        # Dictionary containing values of all the possible options.
        self.dict = {}

    def has_option(self, name, remove=True):
        """ Returns True if argument '--name' was passed on the command
        line. """
        option = f"--{name}"
        count = sys.argv.count(option)
        remove_count = count
        if not remove and count > 0:
            remove_count -= 1
        for _ in range(remove_count):
            sys.argv.remove(option)
        if count > 1:
            _warn_multiple_option(option)
        return count > 0

    def option_value(self, name, short_option_name=None, remove=True):
        """
        Returns the value of a command line option.

        :param name: The name of the command line option.

        :param remove: Whether the option and its value should be
         removed from sys.argv. Useful when there's a need to query for
         the value and also pass it along to setuptools for example.

        :return: Either the option value or None.
        """

        option = f"--{name}"
        short_option = f"-{short_option_name}" if short_option_name else None
        single_option_prefix = f"{option}="
        value = None
        for index in reversed(range(len(sys.argv))):
            arg = sys.argv[index]
            if arg == option or short_option and arg == short_option:
                if value:
                    _warn_multiple_option(option)
                else:
                    if index + 1 >= len(sys.argv):
                        raise RuntimeError(f"The option {option} requires a value")
                    value = sys.argv[index + 1]

                if remove:
                    sys.argv[index:index + 2] = []

            elif arg.startswith(single_option_prefix):
                if value:
                    _warn_multiple_option(option)
                else:
                    value = arg[len(single_option_prefix):]

                if remove:
                    sys.argv[index:index + 1] = []

        self.dict[name] = value
        return value

    def find_qtpaths(self):
        # Skip the first run that will trigger the three different build
        # stated of the setup process
        if self.dict["internal-build-type"] is None:
            return None
        # for these command --qtpaths should not be required
        no_qtpaths_commands = ["--help", "--help-commands", "--qt-target-path", "build_base_docs"]

        for no_qtpaths_command in no_qtpaths_commands:
            if any(no_qtpaths_command in argument for argument in sys.argv):
                return None

        qtpaths = self.option_value("qtpaths")
        if qtpaths is not None:
            return qtpaths

        # if qtpaths is not given as cli option, try to find it in PATH
        qtpaths = which("qtpaths6")
        if qtpaths is not None:
            return str(Path(qtpaths).resolve())

        qtpaths = which("qtpaths")
        if qtpaths is not None:
            return str(Path(qtpaths).resolve())

        if qtpaths is None:
            sys.exit(-1)

        return qtpaths

    def _jobs_option_value(self):
        """Option value for parallel builds."""
        value = self.option_value('parallel', short_option_name='j')

        _deprecated_option_jobs = self.option_value('jobs')
        if _deprecated_option_jobs:
            _warn_deprecated_option('jobs', 'parallel')
            value = _deprecated_option_jobs

        if value:
            return f"-j{value}" if not value.startswith('-j') else value
        return ''

    def resolve(self):
        return {
            "BUILD_TYPE": self.option_value("build-type"),
            "INTERNAL_BUILD_TYPE": self.option_value("internal-build-type"),
            # number of parallel build jobs
            "JOBS": self._jobs_option_value(),
            # Legacy, not used any more.
            "JOM": self.has_option('jom'),
            "MACOS_USE_LIBCPP": self.has_option("macos-use-libc++"),
            "LOG_LEVEL": self.option_value("log-level", remove=False),
            "QUIET": self.has_option('quiet'),
            "VERBOSE_BUILD": self.has_option('verbose-build'),
            "SNAPSHOT_BUILD": self.has_option("snapshot-build"),
            "LIMITED_API": self.option_value("limited-api"),
            "UNOPTIMIZE": self.option_value("unoptimize"),
            "DISABLE_PYI": self.has_option("disable-pyi"),
            "SKIP_MYPY_TEST": self.has_option("skip-mypy-test"),
            "PACKAGE_TIMESTAMP": self.option_value("package-timestamp"),
            # This is used automatically by setuptools.command.install object, to
            # specify the final installation location.
            "FINAL_INSTALL_PREFIX": self.option_value("prefix", remove=False),
            "CMAKE_TOOLCHAIN_FILE": self.option_value("cmake-toolchain-file"),
            "SHIBOKEN_HOST_PATH": self.option_value("shiboken-host-path"),
            "SHIBOKEN_HOST_PATH_QUERY_FILE": self.option_value(
                "internal-shiboken-host-path-query-file"
            ),
            "QT_HOST_PATH": self.option_value("qt-host-path"),
            # This is used to identify the template for doc builds
            "QTPATHS": self.find_qtpaths()
            # This is an optional command line option.
            # If --qtpaths is not provided via command-line,
            # then qtpaths is checked inside PATH variable
        }


class CommandMixin:
    """Mixin for the setuptools build/install commands handling the options."""

    _static_class_finalized_once = False

    mixin_user_options = [
        ('avoid-protected-hack', None, 'Force --avoid-protected-hack'),
        ('debug', None, 'Build with debug information'),
        ('relwithdebinfo', None, 'Build in release mode with debug information'),
        ('only-package', None, 'Package only'),
        ('no-strip', None, 'Do not strip package libraries (release mode)'),
        ('standalone', None, 'Standalone build'),
        ('ignore-git', None, 'Do update subrepositories'),
        ('skip-docs', None, 'Skip documentation build (deprecated)'),
        ('build-docs', None, 'Build the API documentation'),
        ('no-jom', None, 'Do not use jom (MSVC)'),
        ('build-tests', None, 'Build tests'),
        ('use-xvfb', None, 'Use Xvfb for testing'),
        ('reuse-build', None, 'Reuse existing build'),
        ('compiler-launcher=', None, 'Use a compiler launcher like ccache or sccache for builds'),
        ('skip-cmake', None, 'Skip CMake step'),
        ('skip-make-install', None, 'Skip install step'),
        ('skip-packaging', None, 'Skip packaging step'),
        ('log-level=', None, 'Log level of the build.'),
        ('verbose-build', None, 'Verbose build'),
        ('quiet', None, 'Quiet build'),
        ('sanitize-address', None, 'Build with address sanitizer'),
        ('sanitize-thread', None, 'Build with thread sanitizer'),
        ('shorter-paths', None, 'Use shorter paths'),
        ('doc-build-online', None, 'Build online documentation'),
        ('qtpaths=', None, 'Path to qtpaths'),
        ('qmake=', None, 'Path to qmake (deprecated, use qtpaths)'),
        ('qt=', None, 'Qt version'),
        ('qt-target-path=', None,
         'Path to device Qt installation (use Qt libs when cross-compiling)'),
        ('cmake=', None, 'Path to CMake'),
        ('openssl=', None, 'Path to OpenSSL libraries'),

        # FIXME: Deprecated in favor of shiboken-target-path
        ('shiboken-config-dir=', None, 'shiboken configuration directory'),

        ('shiboken-target-path=', None, 'Path to target shiboken package'),
        ('python-target-path=', None, 'Path to target Python installation / prefix'),
        ('make-spec=', None, 'Qt make-spec'),
        ('macos-arch=', None, 'macOS architecture'),
        ('macos-sysroot=', None, 'macOS sysroot'),
        ('macos-deployment-target=', None, 'macOS deployment target'),
        ('skip-modules=', None, 'Qt modules to be skipped'),
        ('module-subset=', None, 'Qt modules to be built'),
        ('rpath=', None, 'RPATH'),
        ('qt-conf-prefix=', None, 'Qt configuration prefix'),
        ('qt-src-dir=', None, 'Qt source directory'),
        ('no-qt-tools', None, 'Do not copy the Qt tools'),
        ('no-size-optimization', None, 'Turn off size optimization for PySide6 binaries'),
        # Default is auto-detected by PysideBuild._enable_numpy()
        ('pyside-numpy-support', None, 'libpyside: Add numpy support (deprecated)'),
        ('enable-numpy-support', None, 'Enable numpy support'),
        ('disable-numpy-support', None, 'Disable numpy support'),
        ('internal-cmake-install-dir-query-file-path=', None,
         'Path to file where the CMake install path of the project will be saved'),

        # We redeclare plat-name as an option so it's recognized by the
        # install command and doesn't throw an error.
        ('plat-name=', None, 'The platform name for which we are cross-compiling'),
        ('unity', None, 'Use CMake UNITY_BUILD_MODE (obsolete)'),
        ('no-unity', None, 'Disable CMake UNITY_BUILD_MODE'),
        ('unity-build-batch-size=', None, 'Value of CMAKE_UNITY_BUILD_BATCH_SIZE'),
        # shiboken-force-process-system-headers option is specifically used to tell the clang
        # inside shiboken to process the system headers, when building against a system Qt.
        #
        # This option is specific for Flatpak and OS distro builds of PySide6. So, use with
        # caution as it may also try to parse other global headers.
        ('shiboken-force-process-system-headers', None,
         'When building PySide against system Qt, shiboken does not ignore the system Qt headers'),
        # shiboken-extra-inlude-paths option is specifically used to tell the clang inside shiboken
        # to include extra paths when parsing the headers. Use with caution.
        ('shiboken-extra-include-paths=', None,
         'Extra include paths for shiboken. Comma separated.'),
        # flatpak option is used to build PySide6 for Flatpak. Flatpak is a special case where
        # some of the headers for the Qt modules are located as system headers in /usr/include in
        # the KDE flatpak SDK. Therefore --shiboken-force-process-system headers will be by
        # default enabled when --flatpak is enabled.
        # Apart from that, headers for certain Qt modules like QtWebEngine, QtPdf etc. are located
        # in /app/include from the Flapak WebEngine baseapp. Therefore when the --flatpak option is
        # enabled, the extra include path of /app/include will be added to the option
        # --shiboken-extra-include-paths.
        ('flatpak', None, 'Build PySide6 for Flatpak.'),
    ]

    def __init__(self):
        self.avoid_protected_hack = False
        self.debug = False
        self.relwithdebinfo = False
        self.no_strip = False
        self.only_package = False
        self.standalone = False
        self.ignore_git = False
        self.skip_docs = False
        self.build_docs = False
        self.no_jom = False
        self.build_tests = False
        self.use_xvfb = False
        self.reuse_build = False
        self.compiler_launcher = None
        self.skip_cmake = False
        self.skip_make_install = False
        self.skip_packaging = False
        self.log_level = "info"
        self.verbose_build = False
        self.sanitize_address = False
        self.sanitize_thread = False
        self.snapshot_build = False
        self.shorter_paths = False
        self.doc_build_online = False
        self.qtpaths = None
        self.qmake = None
        self.has_qmake_option = False
        self.qt = '5'
        self.qt_host_path = None
        self.qt_target_path = None
        self.cmake = None
        self.openssl = None
        self.shiboken_config_dir = None
        self.shiboken_host_path = None
        self.shiboken_host_path_query_file = None
        self.shiboken_target_path = None
        self.python_target_path = None
        self.is_cross_compile = False
        self.cmake_toolchain_file: str = ""
        self.make_spec = None
        self.macos_arch = None
        self.macos_sysroot = None
        self.macos_deployment_target = None
        self.skip_modules = None
        self.module_subset = None
        self.rpath = None
        self.qt_conf_prefix = None
        self.qt_src_dir = None
        self.no_qt_tools = False
        self.no_size_optimization = False
        self.pyside_numpy_support = False
        self.enable_numpy_support = False
        self.disable_numpy_support = False
        self.plat_name = None
        self.internal_cmake_install_dir_query_file_path = None
        self._per_command_mixin_options_finalized = False
        self.unity = False
        self.no_unity = False
        self.unity_build_batch_size = "16"
        self.shiboken_force_process_system_headers = False
        self.shiboken_extra_include_paths = None
        self.flatpak = False

        # When initializing a command other than the main one (so the
        # first one), we need to copy the user options from the main
        # command to the new command options dict. Then
        # Distribution.get_command_obj will pick up the copied options
        # ensuring that all commands that inherit from
        # the mixin, get our custom properties set by the time
        # finalize_options is called.
        if CommandMixin._static_class_finalized_once:
            current_command: Command = self
            dist = current_command.distribution
            main_command_name = dist.commands[0]
            main_command_opts = dist.get_option_dict(main_command_name)
            current_command_name = current_command.get_command_name()
            current_command_opts = dist.get_option_dict(current_command_name)
            mixin_options_set = self.get_mixin_options_set()
            for key, value in main_command_opts.items():
                if key not in current_command_opts and key in mixin_options_set:
                    current_command_opts[key] = value

        # qtpaths is already known before running SetupRunner
        self.qtpaths = OPTION["QTPATHS"]

    @staticmethod
    @memoize
    def get_mixin_options_set():
        keys = set()
        for (name, _, _) in CommandMixin.mixin_user_options:
            keys.add(name.rstrip("=").replace("-", "_"))
        return keys

    def mixin_finalize_options(self):
        # The very first we finalize options, record that.
        if not CommandMixin._static_class_finalized_once:
            CommandMixin._static_class_finalized_once = True

        # Ensure we finalize once per command object, rather than per
        # setup.py invocation. We want to have the option values
        # available in all commands that derive from the mixin.
        if not self._per_command_mixin_options_finalized:
            self._per_command_mixin_options_finalized = True
            self._do_finalize()

    def _do_finalize(self):
        # is_cross_compile must be set before checking for qtpaths/qmake
        # because we DON'T want those to be found when cross compiling.
        # Currently when cross compiling, qt-target-path MUST be used.
        using_cmake_toolchain_file = False
        cmake_toolchain_file: str = ""
        if OPTION["CMAKE_TOOLCHAIN_FILE"]:
            self.is_cross_compile = True
            using_cmake_toolchain_file = True
            cmake_toolchain_file = OPTION["CMAKE_TOOLCHAIN_FILE"]
            self.cmake_toolchain_file = cmake_toolchain_file

        if not self._determine_defaults_and_check():
            sys.exit(-1)
        OPTION['AVOID_PROTECTED_HACK'] = self.avoid_protected_hack
        OPTION['DEBUG'] = self.debug
        OPTION['RELWITHDEBINFO'] = self.relwithdebinfo
        OPTION['NO_STRIP'] = self.no_strip
        OPTION['ONLYPACKAGE'] = self.only_package
        OPTION['STANDALONE'] = self.standalone
        if self.ignore_git:
            _warn_deprecated_option('ignore_git')
        OPTION['SKIP_DOCS'] = self.skip_docs
        OPTION['BUILD_DOCS'] = self.build_docs
        OPTION['BUILDTESTS'] = self.build_tests

        OPTION['NO_JOM'] = self.no_jom
        OPTION['XVFB'] = self.use_xvfb
        OPTION['REUSE_BUILD'] = self.reuse_build
        OPTION['COMPILER_LAUNCHER'] = self.compiler_launcher
        OPTION['SKIP_CMAKE'] = self.skip_cmake
        OPTION['SKIP_MAKE_INSTALL'] = self.skip_make_install
        OPTION['SKIP_PACKAGING'] = self.skip_packaging
        # Logging options:
        # 'quiet' and 'verbose-build' are deprecated,
        # log-level has higher priority when used.
        OPTION['LOG_LEVEL'] = self.log_level
        OPTION['VERBOSE_BUILD'] = self.verbose_build
        # The OPTION["QUIET"] doesn't need to be initialized with a value
        # because is an argument that it will not be removed due to being
        # a setuptools argument as well.

        # By default they are False, so we check if they changed with xor
        if bool(OPTION["QUIET"]) != bool(OPTION["VERBOSE_BUILD"]):
            log.warning("Using --quiet and --verbose-build is deprecated. "
                        "Please use --log-level=quiet or --log-level=verbose instead.")
            # We assign a string value instead of the enum
            # because is what we get from the command line.
            # Later we assign the enum
            if OPTION["QUIET"]:
                OPTION["LOG_LEVEL"] = "quiet"
            elif OPTION["VERBOSE_BUILD"]:
                OPTION["LOG_LEVEL"] = "verbose"

        if OPTION["LOG_LEVEL"] not in ("quiet", "info", "verbose"):
            log.error(f"Invalid value for log level: '--log-level={OPTION['LOG_LEVEL']}'. "
                      "Use 'quiet', 'info', or 'verbose'.")
            sys.exit(-1)
        else:
            if OPTION["LOG_LEVEL"] == "quiet":
                OPTION["LOG_LEVEL"] = LogLevel.QUIET
                log.setLevel(logging.ERROR)
            elif OPTION["LOG_LEVEL"] == "info":
                OPTION["LOG_LEVEL"] = LogLevel.INFO
                log.setLevel(logging.INFO)
            elif OPTION["LOG_LEVEL"] == "verbose":
                OPTION["LOG_LEVEL"] = LogLevel.VERBOSE
                log.setLevel(logging.DEBUG)

        OPTION['SANITIZE_ADDRESS'] = self.sanitize_address
        OPTION['SANITIZE_THREAD'] = self.sanitize_thread
        OPTION['SHORTER_PATHS'] = self.shorter_paths
        OPTION['DOC_BUILD_ONLINE'] = self.doc_build_online
        if self.unity:
            log.warning("Using --unity no longer has any effect, "
                        "Unity build mode is now the default.")
        OPTION['UNITY'] = not self.no_unity
        OPTION['UNITY_BUILD_BATCH_SIZE'] = self.unity_build_batch_size
        OPTION['SHIBOKEN_FORCE_PROCESS_SYSTEM_HEADERS'] = self.shiboken_force_process_system_headers
        OPTION['SHIBOKEN_EXTRA_INCLUDE_PATHS'] = self.shiboken_extra_include_paths
        OPTION['FLATPAK'] = self.flatpak
        if OPTION['FLATPAK']:
            OPTION['SHIBOKEN_FORCE_PROCESS_SYSTEM_HEADERS'] = True
            OPTION['SHIBOKEN_EXTRA_INCLUDE_PATHS'] = '/app/include'

        qtpaths_abs_path = None
        if self.qtpaths and Path(self.qtpaths).exists():
            qtpaths_abs_path = Path(self.qtpaths).resolve()

        # FIXME PYSIDE7: Remove qmake handling
        # make qtinfo.py independent of relative paths.
        qmake_abs_path = None
        if self.qmake:
            qmake_abs_path = Path(self.qmake).resolve()
            OPTION['QMAKE'] = qmake_abs_path
        OPTION['HAS_QMAKE_OPTION'] = self.has_qmake_option
        OPTION['QT_VERSION'] = self.qt
        self.qt_host_path = OPTION['QT_HOST_PATH']
        OPTION['QT_TARGET_PATH'] = self.qt_target_path

        qt_target_path = self.qt_target_path

        # We use the CMake project to find host Qt if neither qmake or
        # qtpaths is available. This happens when building the host
        # tools in the overall cross-building process.
        use_cmake = False
        if (using_cmake_toolchain_file or (not self.qmake
                                           and not self.qtpaths and self.qt_target_path)):
            use_cmake = True

        QtInfo().setup(qtpaths_abs_path, self.cmake, qmake_abs_path,
                       self.has_qmake_option,
                       use_cmake=use_cmake,
                       qt_target_path=qt_target_path,
                       cmake_toolchain_file=cmake_toolchain_file)

        if 'build_base_docs' not in sys.argv:
            try:
                QtInfo().prefix_dir
            except Exception as e:
                if not self.qt_target_path:
                    log.error(
                        "Could not find Qt. You can pass the --qt-target-path=<qt-dir> option "
                        "as a hint where to find Qt.\n")
                else:
                    log.error("Could not find Qt via provided option "
                              f"--qt-target-path={qt_target_path}\n")
                raise e

        OPTION['CMAKE'] = self.cmake.resolve()
        OPTION['OPENSSL'] = self.openssl
        OPTION['SHIBOKEN_CONFIG_DIR'] = self.shiboken_config_dir
        if self.shiboken_config_dir:
            _warn_deprecated_option('shiboken-config-dir', 'shiboken-target-path')

        self.shiboken_host_path = OPTION['SHIBOKEN_HOST_PATH']
        self.shiboken_host_path_query_file = OPTION['SHIBOKEN_HOST_PATH_QUERY_FILE']

        if not self.shiboken_host_path and self.shiboken_host_path_query_file:
            try:
                queried_shiboken_host_path = Path(self.shiboken_host_path_query_file).read_text()
                self.shiboken_host_path = queried_shiboken_host_path
                OPTION['SHIBOKEN_HOST_PATH'] = queried_shiboken_host_path
            except Exception as e:
                log.error(
                    f"\n Could not find shiboken host tools via the query file: "
                    f"{self.shiboken_host_path_query_file:} Error was:\n\n\n")
                raise e

        OPTION['SHIBOKEN_TARGET_PATH'] = self.shiboken_target_path
        OPTION['PYTHON_TARGET_PATH'] = self.python_target_path
        OPTION['MAKESPEC'] = self.make_spec
        OPTION['MACOS_ARCH'] = self.macos_arch
        OPTION['MACOS_SYSROOT'] = self.macos_sysroot
        OPTION['MACOS_DEPLOYMENT_TARGET'] = self.macos_deployment_target
        OPTION['SKIP_MODULES'] = self.skip_modules
        OPTION['MODULE_SUBSET'] = self.module_subset
        OPTION['RPATH_VALUES'] = self.rpath
        OPTION['QT_CONF_PREFIX'] = self.qt_conf_prefix
        OPTION['QT_SRC'] = self.qt_src_dir
        OPTION['NO_QT_TOOLS'] = self.no_qt_tools
        OPTION['NO_OVERRIDE_OPTIMIZATION_FLAGS'] = self.no_size_optimization
        OPTION['DISABLE_NUMPY_SUPPORT'] = self.disable_numpy_support
        OPTION['ENABLE_NUMPY_SUPPORT'] = self.enable_numpy_support
        OPTION['PYSIDE_NUMPY_SUPPORT'] = self.pyside_numpy_support

        if not self._extra_checks():
            sys.exit(-1)

        OPTION['PLAT_NAME'] = self.plat_name

    def _extra_checks(self):
        if self.is_cross_compile and not self.plat_name:
            log.error("No value provided to --plat-name while cross-compiling.")
            return False
        return True

    def _determine_defaults_and_check(self):
        if not self.cmake:
            self.cmake = Path(which("cmake"))
        elif isinstance(self.cmake, str):  # command line option
            self.cmake = Path(self.cmake)
        if not self.cmake:
            log.error("cmake could not be found.")
            return False
        if not self.cmake.exists():
            log.error(f"'{self.cmake}' does not exist.")
            return False

        # Setting up the Paths when passing via command line
        if isinstance(self.qtpaths, str):
            self.qtpaths = Path(self.qtpaths)
        if isinstance(self.qmake, str):
            self.qmake = Path(self.qmake)
        if self.qt_target_path and isinstance(self.qt_target_path, str):
            self.qt_target_path = Path(self.qt_target_path)

        # When cross-compiling, we only accept the qt-target-path
        # option and don't rely on auto-searching in PATH or the other
        # qtpaths / qmake options.
        # We also don't do auto-searching if qt-target-path is passed
        # explicitly. This is to help with the building of host tools
        # while cross-compiling.
        # Skip this process for the 'build_base_docs' command
        if (not self.is_cross_compile
            and not self.qt_target_path
                and 'build_base_docs' not in sys.argv):
            # Enforce usage of qmake in QtInfo if it was given explicitly.
            if self.qmake:
                self.has_qmake_option = True
                _warn_deprecated_option('qmake', 'qtpaths')

            # If no tool was specified and qtpaths was not found in PATH,
            # ask to provide a path to qtpaths.
            if not self.qtpaths and not self.qmake and not self.qt_target_path:
                log.error("No value provided to --qtpaths option. Please provide one to find Qt.")
                return False

            # Validate that the given tool path exists.
            if self.qtpaths and not self.qtpaths.exists():
                log.error(f"The specified qtpaths path '{self.qtpaths}' does not exist.")
                return False

            if self.qmake and not self.qmake.exists():
                log.error(f"The specified qmake path '{self.qmake}' does not exist.")
                return False
        else:
            # Check for existence, but don't require if it's not set. A
            # check later will be done to see if it's needed.
            if self.qt_target_path and not self.qt_target_path.exists():
                log.error(f"Provided --qt-target-path='{self.qt_target_path}' "
                          "path does not exist.")
                return False

        if not self.make_spec:
            self.make_spec = _AVAILABLE_MKSPECS[0]
        if self.make_spec not in _AVAILABLE_MKSPECS:
            log.error(f'Invalid option --make-spec "{self.make_spec}". '
                      f'Available values are {_AVAILABLE_MKSPECS}')
            return False

        if OPTION["JOBS"] and sys.platform == 'win32' and self.no_jom:
            log.error("Option --jobs can only be used with jom on Windows.")
            return False

        if sys.platform == 'win32' and OPTION["LIMITED_API"] == "yes" and self.debug:
            log.error("It is not possible to make a debug build of PySide6 with limited API. "
                      "Please select a release build or disable limited API.")
            return False

        return True


# OPTION dictionary that will be imported in other build_scripts
OPTION = Options().resolve()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations


import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


class SignalManagerRefCount(unittest.TestCase):
    """Simple test case to check if the signal_manager is erroneously incrementing the
       object refcounter."""

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testObjectRefcount(self):
        """Emission of QObject.destroyed() to a python slot"""
        def callback():
            pass
        obj = QObject()
        refcount = sys.getrefcount(obj)
        obj.destroyed.connect(callback)
        self.assertEqual(refcount, sys.getrefcount(obj))
        obj.destroyed.disconnect(callback)
        self.assertEqual(refcount, sys.getrefcount(obj))


if __name__ == '__main__':
    unittest.main()
